@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CatJob.cc,v 1.16 2000/01/09 15:06:00 lav Exp $ */
+/* $Id: CatJob.cc,v 1.18 2000/07/19 07:24:14 lav Exp $ */
 
 #include <config.h>
 #include <stdio.h>
@@ -67,15 +67,7 @@ void CatJob::NextFile()
       return;
    }
 
-   ParsedURL src_url(src,true);
-   FileCopyPeerFA *src_peer=0;
-   if(src_url.proto==0)
-   {
-      src_peer=new FileCopyPeerFA(session,src,FA::RETRIEVE);
-      src_peer->DontReuseSession();
-   }
-   else
-      src_peer=new FileCopyPeerFA(&src_url,FA::RETRIEVE);
+   FileCopyPeerFA *src_peer=FileCopyPeerFA::New(session,src,FA::RETRIEVE,false);
 
    FileCopyPeerFDStream *dst_peer=0;
    if(for_each)
@@ -93,7 +85,7 @@ void CatJob::NextFile()
 
    FileCopy *copier=FileCopy::New(src_peer,dst_peer,false);
    copier->DontCopyDate();
-   if(ascii)
+   if(ascii || (auto_ascii && (isatty(global->getfd()) || global->usesfd(1))))
    {
       if(global->usesfd(1))
 	 copier->LineBuffered();
@@ -143,6 +135,7 @@ CatJob::CatJob(FileAccess *new_session,FDStream *new_global,ArgV *new_args)
    global=new_global;
    for_each=0;
    ascii=false;
+   auto_ascii=true;
 
    if(!strcmp(op,"more") || !strcmp(op,"zmore"))
    {
@@ -155,7 +148,10 @@ CatJob::CatJob(FileAccess *new_session,FDStream *new_global,ArgV *new_args)
       }
    }
    if(!strcmp(op,"zcat") || !strcmp(op,"zmore"))
+   {
       for_each="zcat";
+      Binary();
+   }
 
    if(!global)
    {
