@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.h,v 1.33.2.1 2000/06/08 10:13:30 lav Exp $ */
+/* $Id: CmdExec.h,v 1.38 2000/08/25 23:19:28 lav Exp $ */
 
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
@@ -48,6 +48,7 @@ public:
    virtual ~CmdFeeder() {}
 
    virtual void clear() {}
+   virtual bool RealEOF() { return true; }
 };
 
 extern CmdFeeder *lftp_feeder;	 // feeder to use after 'lftp' command
@@ -109,7 +110,7 @@ private:
 
    enum builtins
    {
-      BUILTIN_NONE,
+      BUILTIN_NONE=0,
       BUILTIN_OPEN,
       BUILTIN_CD,
       BUILTIN_EXEC_RESTART,
@@ -123,6 +124,8 @@ private:
    GlobURL *glob;
    ArgV *args_glob;
 
+   int redirections;
+
    static CmdExec *chain;
    CmdExec *next;
 
@@ -145,7 +148,7 @@ public:
    void FeedQuoted(const char *c);
    void AtExit();
 
-   void SuspendJob();
+   void SuspendJob(Job *j);
 
    CmdExec(FileAccess *s);
    ~CmdExec();
@@ -221,6 +224,7 @@ public:
    static const cmd_rec *CmdByIndex(int i);
 
    int	 last_bg;
+   bool	 wait_all;
 
    void pre_stdout();
 };
