@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.61.2.1 2000/07/31 16:42:19 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.75 2000/09/12 13:47:48 lav Exp $ */
 
 #include <config.h>
 
@@ -50,6 +50,7 @@ FileAccess *FileAccess::chain=0;
 
 void FileAccess::Init()
 {
+   vproto=0;
    home=0;
    portname=0;
    hostname=0;
@@ -58,6 +59,7 @@ void FileAccess::Init()
    pass_open=false;
 
    file=0;
+   file_url=0;
    file1=0;
    real_cwd=0;
    default_cwd="~";
@@ -66,6 +68,7 @@ void FileAccess::Init()
    pos=0;
    mode=CLOSED;
    try_time=0;
+   retries=0;
    time(&event_time);
    opt_date=0;
    opt_size=0;
@@ -75,6 +78,7 @@ void FileAccess::Init()
    mkdir_p=false;
    ascii=false;
    norest_manual=false;
+   location=0;
 
    url=0;
 
@@ -82,6 +86,7 @@ void FileAccess::Init()
    entity_date=NO_DATE;
 
    closure=0;
+   res_prefix=0;
 
    chmod_mode=0644;
 
@@ -116,19 +121,22 @@ FileAccess::FileAccess(const FileAccess *fa)
 
 FileAccess::~FileAccess()
 {
-   xfree(file); file=0;
-   xfree(cwd); cwd=0;
-   xfree(real_cwd); real_cwd=0;
-   xfree(error); error=0;
-   xfree(home); home=0;
-   xfree(user); user=0;
-   xfree(pass); pass=0;
-   xfree(group); group=0;
-   xfree(gpass); gpass=0;
-   xfree(hostname); hostname=0;
-   xfree(portname); portname=0;
-   xfree(url); url=0;
-   xfree(closure); closure=0;
+   xfree(vproto);
+   xfree(file);
+   xfree(file_url);
+   xfree(cwd);
+   xfree(real_cwd);
+   xfree(error);
+   xfree(home);
+   xfree(user);
+   xfree(pass);
+   xfree(group);
+   xfree(gpass);
+   xfree(hostname);
+   xfree(portname);
+   xfree(url);
+   xfree(closure);
+   xfree(location);
    for(FileAccess **scan=&chain; *scan; scan=&((*scan)->next))
    {
       if(*scan==this)
@@ -244,7 +252,7 @@ void FileAccess::CloseOnExec(int fd)
 }
 
 
-void  FileAccess::Open(const char *fn,int mode,long offs)
+void  FileAccess::Open(const char *fn,int mode,off_t offs)
 {
 #ifdef OPEN_DEBUG
    printf("FA::Open(%s)\n",fn);
@@ -257,7 +265,6 @@ void  FileAccess::Open(const char *fn,int mode,long offs)
    pos=offs;
    this->mode=mode;
    mkdir_p=false;
-   try_time=0;
    Timeout(0);
 
    switch((open_mode)mode)
@@ -265,6 +272,7 @@ void  FileAccess::Open(const char *fn,int mode,long offs)
    case STORE:
    case REMOVE:
    case MAKE_DIR:
+   case CHANGE_MODE:
       LsCache::FileChanged(this,file);
       break;
    case REMOVE_DIR:
@@ -281,6 +289,12 @@ const char *FileAccess::StrError(int err)
    static char *str=0;
    static unsigned str_allocated=0;
 
+   if(error)
+   {
+      if(str_allocated<128+strlen(error))
+	 str=(char*)xrealloc(str,str_allocated=128+strlen(error));
+   }
+
    // note to translators: several errors should not be displayed to user;
    // so no need to translate them.
    switch(err)
@@ -292,10 +306,7 @@ const char *FileAccess::StrError(int err)
    case(SEE_ERRNO):
       if(error)
       {
-	 const char *e=strerror(saved_errno);
-	 if(str_allocated<strlen(e)+64+strlen(error))
-	    str=(char*)xrealloc(str,str_allocated=strlen(e)+64+strlen(error));
-   	 sprintf(str,"%s: %s",error,e);
+   	 sprintf(str,"%s: %s",error,strerror(saved_errno));
 	 return(str);
       }
       return(strerror(saved_errno));
@@ -306,8 +317,6 @@ const char *FileAccess::StrError(int err)
    case(NO_FILE):
       if(error)
       {
-	 if(str_allocated<64+strlen(error))
-	    str=(char*)xrealloc(str,64+strlen(error));
    	 sprintf(str,_("Access failed: %s"),error);
 	 return(str);
       }
@@ -317,8 +326,6 @@ const char *FileAccess::StrError(int err)
    case(FATAL):
       if(error)
       {
-	 if(str_allocated<64+strlen(error))
-	    str=(char*)xrealloc(str,str_allocated=64+strlen(error));
    	 sprintf(str,_("Fatal error: %s"),error);
 	 return(str);
       }
@@ -326,9 +333,26 @@ const char *FileAccess::StrError(int err)
    case(STORE_FAILED):
       return(_("Store failed - you have to reput"));
    case(LOGIN_FAILED):
+      if(error)
+      {
+   	 sprintf(str,"%s: %s",_("Login failed"),error);
+	 return(str);
+      }
       return(_("Login failed"));
    case(NOT_SUPP):
       return(_("Operation not supported"));
+   case(FILE_MOVED):
+      if(error)
+      {
+   	 sprintf(str,_("File moved: %s"),error);
+      }
+      else
+      {
+	 if(str_allocated<64+xstrlen(location))
+	    str=(char*)xrealloc(str,64+xstrlen(location));
+	 sprintf(str,_("File moved to `%s'"),location?location:"?");
+      }
+      return str;
    }
    return("");
 }
@@ -336,6 +360,7 @@ const char *FileAccess::StrError(int err)
 void FileAccess::Close()
 {
    xfree(file); file=0;
+   xfree(file_url); file_url=0;
    xfree(file1); file1=0;
    mode=CLOSED;
    opt_date=0;
@@ -345,6 +370,7 @@ void FileAccess::Close()
    entity_date=NO_DATE;
    ascii=false;
    norest_manual=false;
+   xfree(location); location=0;
    ClearError();
 }
 
@@ -379,7 +405,7 @@ bool FileAccess::SameSiteAs(FileAccess *fa)
 const char *FileAccess::GetFileURL(const char *f,int flags)
 {
    int len=1;
-   const char *proto=GetProto();
+   const char *proto=GetVisualProto();
    if(proto[0]==0)
       return "";
 
@@ -593,7 +619,11 @@ void FileAccess::Chdir(const char *path,bool verify)
    ExpandTildeInCWD();
 
    if(cwd[0]=='/')
+   {
       prefix_size=1;
+      if(cwd[1]=='/' && cwd[2] && cwd[2]!='/')
+	 prefix_size=2;
+   }
    else if(cwd[0]=='~')
    {
       prefix_size=1;
@@ -612,6 +642,8 @@ void FileAccess::Chdir(const char *path,bool verify)
    {
       strcpy(newcwd,path);
       prefix_size=1;
+      if(path[1]=='/' && path[2] && path[2]!='/')
+	 prefix_size=2;
    }
    else if(path[0]=='~')
    {
@@ -717,6 +749,12 @@ void FileAccess::SetError(int ec,const char *e)
 {
    if(ec==SEE_ERRNO)
       saved_errno=errno;
+   if(ec==NO_FILE && file && file[0] && !strstr(e,file))
+   {
+      char *m=string_alloca(strlen(e)+2+strlen(file)+2);
+      sprintf(m,"%s (%s)",e,file);
+      e=m;
+   }
    xfree(error);
    error=xstrdup(e);
    error_code=ec;
@@ -852,7 +890,9 @@ void FileAccess::BumpEventTime(time_t t)
 
 ResValue FileAccess::Query(const char *name,const char *closure)
 {
-   const char *prefix=GetProto();
+   const char *prefix=res_prefix;
+   if(!prefix)
+      prefix=GetProto();
    char *fullname=(char*)alloca(3+strlen(prefix)+1+strlen(name)+1);
    sprintf(fullname,"%s:%s",prefix,name);
    return ResMgr::Query(fullname,closure);
@@ -904,9 +944,27 @@ FileAccess *FileAccess::NextSameSite(FA *scan)
 }
 
 
+FileAccess *FileAccess::New(const char *proto,const char *host)
+{
+   const char *vproto=proto;
+   // very special case for ftp vs hftp
+   if(!strcmp(proto,"ftp"))
+   {
+      const char *proxy=ResMgr::Query("ftp:proxy",host);
+      if(proxy && !strncmp(proxy,"http://",7))
+      {
+	 vproto=proto;
+	 proto="hftp";
+      }
+   }
+   FA *session=Protocol::NewSession(proto);
+   if(session && vproto!=proto)
+      session->SetVisualProto(vproto);
+   return session;
+}
 FileAccess *FileAccess::New(const ParsedURL *u,bool dummy)
 {
-   FileAccess *s=New(u->proto);
+   FileAccess *s=New(u->proto,u->host);
    if(!s)
    {
       if(!dummy)
@@ -1015,48 +1073,28 @@ void ListInfo::SetExclude(const char *p,regex_t *x,regex_t *i)
 // Glob implementation
 Glob::~Glob()
 {
-   free_list();
    xfree(pattern);
 }
 
 Glob::Glob(const char *p)
 {
    pattern=xstrdup(p);
-   list=0;
-   list_size=0;
-   list_alloc=0;
    dirs_only=false;
    files_only=false;
    match_period=true;
 }
 
-void Glob::free_list()
-{
-   for(int i=0; i<list_size; i++)
-      xfree(list[i]);
-   xfree(list);
-   list=0;
-   list_alloc=0;
-   list_size=0;
-}
-
 void Glob::add_force(const char *ptr,int len)
 {
    // insert new file name into list
-   if(list_size>=list_alloc-1)
-   {
-      if(list_alloc==0)
-	 list_alloc=32;
-      list=(char**)xrealloc(list,(list_alloc*=2)*sizeof(*list));
-   }
-   list[list_size]=(char*)xmalloc(len+1);
-   memcpy(list[list_size],ptr,len);
-   list[list_size][len]=0;
-   list[++list_size]=0;
+   char *s=string_alloca(len+1);
+   memcpy(s,ptr,len);
+   s[len]=0;
+   list.Add(new FileInfo(s));
 }
 void Glob::add(const char *ptr,int len)
 {
-   char *s=(char*)alloca(len+1);
+   char *s=string_alloca(len+1);
    memcpy(s,ptr,len);
    s[len]=0;
 
@@ -1070,6 +1108,36 @@ void Glob::add(const char *ptr,int len)
 
    add_force(ptr,len);
 }
+void Glob::add_force(const FileInfo *info)
+{
+   // insert new file name into list
+   list.Add(new FileInfo(*info));
+}
+void Glob::add(const FileInfo *info)
+{
+   if(info->defined&info->TYPE)
+   {
+      if(dirs_only && info->filetype==info->NORMAL)
+	 return;   // note that symlinks can point to directories,
+		   // so skip normal files only.
+      if(files_only && info->filetype==info->DIRECTORY)
+	 return;
+   }
+
+   char *s=info->name;
+   if(s==0)
+      return;
+
+   int flags=FNM_PATHNAME;
+   if(match_period)
+      flags|=FNM_PERIOD;
+
+   if(pattern[0]!=0
+   && fnmatch(pattern, s, flags)!=0)
+      return; // unmatched
+
+   add_force(info);
+}
 
 bool Glob::HasWildcards(const char *s)
 {
@@ -1165,18 +1233,45 @@ GlobURL::~GlobURL()
       SessionPool::Reuse(session);
    xfree(url_prefix);
 }
-char **GlobURL::GetResult()
+FileSet *GlobURL::GetResult()
 {
-   char **list=glob->GetResult();
-   if(!list)
-      return 0;
+   FileSet &list=*glob->GetResult();
    if(!reuse)
-      return list;
+      return &list;
    for(int i=0; list[i]; i++)
-   {
-      const char *n=url_file(url_prefix,list[i]);
-      list[i]=(char*)xrealloc(list[i],strlen(n)+1);
-      strcpy(list[i],n);
-   }
-   return list;
+      list[i]->SetName(url_file(url_prefix,list[i]->name));
+   return &list;
+}
+
+#include "modconfig.h"
+#ifndef MODULE_PROTO_FTP
+# include "ftpclass.h"
+# define _ftp Ftp::ClassInit()
+#else
+# define _ftp
+#endif
+#ifndef MODULE_PROTO_FILE
+# include "LocalAccess.h"
+# define _file LocalAccess::ClassInit()
+#else
+# define _file
+#endif
+#ifndef MODULE_PROTO_HTTP
+# include "Http.h"
+# define _http Http::ClassInit()
+#else
+# define _http
+#endif
+#ifndef MODULE_PROTO_FISH
+# include "Fish.h"
+# define _fish Fish::ClassInit()
+#else
+# define _fish
+#endif
+void FileAccess::ClassInit()
+{
+   _ftp;
+   _file;
+   _http;
+//    _fish;
 }
