@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.h,v 1.45 2000/03/07 00:34:05 lav Exp $ */
+/* $Id: FileAccess.h,v 1.53 2000/07/30 15:54:31 lav Exp $ */
 
 #ifndef FILEACCESS_H
 #define FILEACCESS_H
@@ -40,6 +40,7 @@
 
 #include "xmalloc.h"
 #include "ResMgr.h"
+#include "FileSet.h"
 
 #define NO_SIZE	     (-1L)
 #define NO_SIZE_YET  (-2L)
@@ -76,7 +77,7 @@ public:
    struct fileinfo
    {
       const char *file;
-      long size;
+      off_t size;
       time_t time;
       bool get_size:1;
       bool get_time:1;
@@ -84,6 +85,7 @@ public:
    };
 
 protected:
+   char	 *vproto;
    char	 *hostname;
    char	 *portname;
    char  *user;
@@ -96,13 +98,14 @@ protected:
    const char *default_cwd;
    char  *cwd;
    char  *file;
+   char	 *file_url;
    char	 *file1;
    int	 mode;
-   long	 pos;
-   long	 real_pos;
+   off_t pos;
+   off_t real_pos;
 
    time_t *opt_date;
-   long	  *opt_size;
+   off_t  *opt_size;
 
    int	 Poll(int fd,int ev);
    int   CheckHangup(struct pollfd *pfd,int num);
@@ -114,6 +117,7 @@ protected:
    time_t   try_time;
    time_t   event_time;
    void BumpEventTime(time_t t);
+   int	 retries;
 
    fileinfo *array_for_info;
    int	 array_ptr;
@@ -135,10 +139,11 @@ protected:
 
    char *url;
 
-   long   entity_size; // size of file to be sent
+   off_t  entity_size; // size of file to be sent
    time_t entity_date; // date of file to be sent
 
    char *closure;
+   const char *res_prefix;
    ResValue Query(const char *name,const char *closure=0);
 
    int chmod_mode;
@@ -153,6 +158,7 @@ protected:
    void Fatal(const char *mess);
    char *error;
    int error_code;
+   char *location;
 
    FileAccess *next;
    static FileAccess *chain;
@@ -168,6 +174,8 @@ public:
    bool SameProtoAs(FileAccess *fa) { return !strcmp(GetProto(),fa->GetProto()); }
    virtual FileAccess *Clone() = 0;
 
+   const char *GetVisualProto() { return vproto?vproto:GetProto(); }
+   void SetVisualProto(const char *p) { xfree(vproto); vproto=xstrdup(p); }
    const char  *GetHome() { return home; }
    const char  *GetHostName() { return hostname; }
    const char  *GetUser() { return user; }
@@ -184,11 +192,12 @@ public:
    virtual void Login(const char *u,const char *p);
    virtual void GroupLogin(const char *g,const char *p);
 
-   virtual void Open(const char *file,int mode,long pos=0);
-   void SetSize(long s) { entity_size=s; }
+   virtual void Open(const char *file,int mode,off_t pos=0);
+   void SetFileURL(const char *u) { xfree(file_url); file_url=xstrdup(u); }
+   void SetSize(off_t s) { entity_size=s; }
    void SetDate(time_t d) { entity_date=d; }
    void WantDate(time_t *d) { opt_date=d; }
-   void WantSize(long *s) { opt_size=s; }
+   void WantSize(off_t *s) { opt_size=s; }
    void AsciiTransfer() { ascii=true; }
    virtual void Close();
 
@@ -214,8 +223,8 @@ public:
    virtual int Buffered();
    virtual int StoreStatus() = 0;
    virtual bool IOReady();
-   long GetPos() { return pos; }
-   long GetRealPos() { return real_pos<0?pos:real_pos; }
+   off_t GetPos() { return pos; }
+   off_t GetRealPos() { return real_pos<0?pos:real_pos; }
    void SeekReal() { pos=GetRealPos(); }
    void RereadManual() { norest_manual=true; }
 
@@ -253,6 +262,7 @@ public:
       NOT_OPEN,
       NO_FILE,
       NO_HOST,
+      FILE_MOVED,
       FATAL,
       STORE_FAILED,
       LOGIN_FAILED,
@@ -271,6 +281,8 @@ public:
 
    static bool NotSerious(int err);
 
+   const char *GetNewLocation() { return location; }
+
    void Reconfig(const char *);
 
 
@@ -293,10 +305,7 @@ public:
 	 (void)new Protocol(proto,creator);
       }
 
-   static FileAccess *New(const char *proto)
-      {
-	 return Protocol::NewSession(proto);
-      }
+   static FileAccess *New(const char *proto,const char *host=0);
    static FileAccess *New(const class ParsedURL *u,bool dummy=true);
 
    void SetPasswordGlobal(const char *p);
@@ -311,6 +320,14 @@ public:
 	 priority=p;
 	 current->Timeout(0);
       }
+
+   // not pretty (FIXME)
+   int GetRetries() { return retries; }
+   void SetRetries(int r) { retries=r; }
+   time_t GetTryTime() { return try_time; }
+   void SetTryTime(time_t t) { try_time=t; }
+
+   static void ClassInit();
 };
 
 class FileAccessOperation : public SMTask
@@ -341,24 +358,24 @@ class Glob : public FileAccessOperation
 {
 protected:
    char  *pattern;
-   char	 **list;
-   int	 list_size;
-   int	 list_alloc;
+   FileSet list;
    bool	 dirs_only;
    bool	 files_only;
    bool	 match_period;
    void	 add(const char *ptr,int len);
    void	 add_force(const char *ptr,int len);
    void	 add(const char *ptr) { add(ptr,strlen(ptr)); }
-   void	 free_list();
+   void	 add(const FileInfo *info);
+   void	 add_force(const FileInfo *info);
    virtual ~Glob();
 public:
    const char *GetPattern() { return pattern; }
-   char **GetResult() { return list; }
+   FileSet *GetResult() { return &list; }
    Glob(const char *p);
    void DirectoriesOnly() { dirs_only=true; }
    void FilesOnly() { files_only=true; }
    void NoMatchPeriod() { match_period=false; }
+   void SortByName() { list.SortByName(); }
 
    static bool HasWildcards(const char *);
    static void UnquoteWildcards(char *);
@@ -380,11 +397,12 @@ public:
    Glob *glob;
    GlobURL(FileAccess *s,const char *p);
    ~GlobURL();
-   char **GetResult();
+   FileSet *GetResult();
    bool Done()  { return glob->Done(); }
    bool Error() { return glob->Error(); }
    const char *ErrorText() { return glob->ErrorText(); }
    const char *Status() { return glob->Status(); }
+   void SortByName() { glob->SortByName(); }
 };
 
 #include "FileSet.h"
