@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.55.2.1 2000/06/08 10:13:30 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.70 2000/09/29 07:51:21 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -45,12 +45,14 @@
 #include "log.h"
 #include "misc.h"
 #include "LsCache.h"
+#include "plural.h"
 
 #define skip_threshold 0x1000
 #define debug(a) Log::global->Format a
 
-ResDecl rate_period  ("xfer:rate-period","15", ResMgr::UNumberValidate,0);
-ResDecl eta_period   ("xfer:eta-period", "120",ResMgr::UNumberValidate,0);
+ResDecl rate_period  ("xfer:rate-period","15", ResMgr::UNumberValidate,ResMgr::NoClosure);
+ResDecl eta_period   ("xfer:eta-period", "120",ResMgr::UNumberValidate,ResMgr::NoClosure);
+ResDecl max_redir    ("xfer:max-redirections", "0",ResMgr::UNumberValidate,ResMgr::NoClosure);
 
 // FileCopy
 #define super SMTask
@@ -139,7 +141,7 @@ int FileCopy::Do()
       }
       if(put->Broken())
       {
-	 debug((9,"copy: put is broken\n"));
+	 debug((9,_("copy: put is broken\n")));
 	 goto pre_GET_DONE_WAIT;
       }
       put->Resume();
@@ -153,15 +155,15 @@ int FileCopy::Do()
 			      || put->GetRealPos()<put->range_start
 			      || get->GetRealPos()!=put->GetRealPos()))
       {
-	 SetError("seek failed");
+	 SetError(_("seek failed"));
 	 return MOVED;
       }
       long lbsize=0;
       if(line_buffer)
 	 lbsize=line_buffer->Size();
       /* check if positions are correct */
-      long get_pos=get->GetRealPos()-get->range_start;
-      long put_pos=put->GetRealPos()-put->range_start;
+      off_t get_pos=get->GetRealPos()-get->range_start;
+      off_t put_pos=put->GetRealPos()-put->range_start;
       if(get_pos-lbsize!=put_pos)
       {
 	 if(line_buffer)
@@ -175,25 +177,26 @@ int FileCopy::Do()
 	    if(!get->CanSeek(put->GetRealPos()))
 	    {
 	       // we lose... How about a large buffer?
-	       SetError("cannot seek on data source");
+	       SetError(_("cannot seek on data source"));
 	       return MOVED;
 	    }
-	    debug((9,"copy: put rolled back to %ld, seeking get accordingly\n",
-		     put->GetRealPos()));
-	    debug((10,"copy: get position was %ld\n",get->GetRealPos()));
+	    debug((9,_("copy: put rolled back to %lld, seeking get accordingly\n"),
+		     (long long)put->GetRealPos()));
+	    debug((10,_("copy: get position was %lld\n"),
+		     (long long)get->GetRealPos()));
 	    get->Seek(put->GetRealPos());
 	    return MOVED;
 	 }
 	 else // put_pos > get_pos
 	 {
-	    long size=get->GetSize();
+	    off_t size=get->GetSize();
 	    if(size>=0 && put->GetRealPos()>=size)
 	    {
 	       // simulate eof, as we have already have the whole file.
-	       debug((9,"copy: all data received, but get rolled back\n"));
+	       debug((9,_("copy: all data received, but get rolled back\n")));
 	       goto eof;
 	    }
-	    int skip=put->GetRealPos()-get->GetRealPos();
+	    off_t skip=put->GetRealPos()-get->GetRealPos();
 	    if(!put->CanSeek(get->GetRealPos()) || skip<skip_threshold)
 	    {
 	       // we have to skip some data
@@ -202,13 +205,12 @@ int FileCopy::Do()
 		  skip=s;
 	       if(skip==0)
 		  return m;
-	       debug((9,"copy: skipping %d bytes on get to adjust to put\n",skip));
 	       get->Skip(skip);
 	       bytes_count+=skip;
 	       return MOVED;
 	    }
-	    debug((9,"copy: get rolled back to %ld, seeking put accordingly\n",
-		     get->GetRealPos()));
+	    debug((9,_("copy: get rolled back to %lld, seeking put accordingly\n"),
+		     (long long)get->GetRealPos()));
 	    put->Seek(get->GetRealPos());
 	    return MOVED;
 	 }
@@ -218,8 +220,9 @@ int FileCopy::Do()
       get->Get(&b,&s);
       if(b==0) // eof
       {
-	 debug((10,"copy: get hit eof\n"));
+	 debug((10,_("copy: get hit eof\n")));
       eof:
+	 put->Resume();
 	 if(line_buffer)
 	 {
 	    line_buffer->Get(&b,&s);
@@ -290,7 +293,7 @@ int FileCopy::Do()
 
       if(get->range_limit!=FILE_END && get->range_limit<=get->GetRealPos())
       {
-	 debug((10,"copy: get reached range limit\n"));
+	 debug((10,_("copy: get reached range limit\n")));
 	 goto eof;
       }
       return m;
@@ -312,7 +315,7 @@ int FileCopy::Do()
 
       if(!put->Done())
 	 return m;
-      debug((10,"copy: put confirmed store\n"));
+      debug((10,_("copy: put confirmed store\n")));
 
    pre_GET_DONE_WAIT:
       get->Empty();
@@ -334,7 +337,7 @@ int FileCopy::Do()
       }
       if(!get->Done())
 	 return m;
-      debug((10,"copy: get is finished - all done\n"));
+      debug((10,_("copy: get is finished - all done\n")));
       state=ALL_DONE;
       Delete(get); get=0;
       return MOVED;
@@ -453,7 +456,7 @@ void FileCopy::LineBuffered(int s)
    line_buffer_max=s;
 }
 
-long FileCopy::GetPos()
+off_t FileCopy::GetPos()
 {
    if(put)
       return put->GetRealPos() - put->Buffered();
@@ -462,7 +465,7 @@ long FileCopy::GetPos()
    return 0;
 }
 
-long FileCopy::GetSize()
+off_t FileCopy::GetSize()
 {
    if(get)
       return get->GetSize();
@@ -473,15 +476,15 @@ int FileCopy::GetPercentDone()
 {
    if(!get || !put)
       return 100;
-   long size=get->GetSize();
+   off_t size=get->GetSize();
    if(size==NO_SIZE || size==NO_SIZE_YET)
       return -1;
    if(size==0)
       return 0;
-   long ppos=put->GetRealPos() - put->Buffered() - put->range_start;
+   off_t ppos=put->GetRealPos() - put->Buffered() - put->range_start;
    if(ppos<0)
       return 0;
-   long psize=size-put->range_start;
+   off_t psize=size-put->range_start;
    if(put->range_limit!=FILE_END)
       psize=put->range_limit-put->range_start;
    if(psize<0)
@@ -509,13 +512,13 @@ const char *FileCopy::GetRateStr()
       return "";
    return rate->GetStrS();
 }
-long FileCopy::GetBytesRemaining()
+off_t FileCopy::GetBytesRemaining()
 {
    if(!get)
       return 0;
    if(get->range_limit==FILE_END)
    {
-      long size=get->GetSize();
+      off_t size=get->GetSize();
       if(size<=0 || size<get->GetRealPos() || !rate_for_eta->Valid())
 	 return -1;
       return(size-GetPos());
@@ -524,16 +527,16 @@ long FileCopy::GetBytesRemaining()
 }
 const char *FileCopy::GetETAStr()
 {
-   long b=GetBytesRemaining();
+   off_t b=GetBytesRemaining();
    if(b<0 || !put)
       return "";
    return rate_for_eta->GetETAStrSFromSize(b);
 }
-long FileCopy::GetETA(long b)
+long FileCopy::GetETA(off_t b)
 {
    if(b<0 || !rate_for_eta->Valid())
       return -1;
-   return (long)(b / rate_for_eta->Get() + 0.5);
+   return (long)(double(b) / rate_for_eta->Get() + 0.5);
 }
 const char *FileCopy::GetStatus()
 {
@@ -597,7 +600,7 @@ void FileCopy::Kill(int sig)
 // FileCopyPeer implementation
 #undef super
 #define super Buffer
-void FileCopyPeer::SetSize(long s)
+void FileCopyPeer::SetSize(off_t s)
 {
    want_size=false;
    size=s;
@@ -748,6 +751,8 @@ int FileCopyPeerFA::Do()
 		  return m;
 	       if(res==FA::STORE_FAILED)
 	       {
+		  try_time=session->GetTryTime();
+		  retries=session->GetRetries();
 		  session->Close();
 		  if(can_seek && seek_pos>0)
 		     Seek(FILE_END);
@@ -816,7 +821,7 @@ const char *FileCopyPeerFA::GetStatus()
    return session->CurrentStatus();
 }
 
-void FileCopyPeerFA::Seek(long new_pos)
+void FileCopyPeerFA::Seek(off_t new_pos)
 {
    if(pos==new_pos)
       return;
@@ -836,7 +841,8 @@ void FileCopyPeerFA::OpenSession()
       if(size!=NO_SIZE && size!=NO_SIZE_YET && seek_pos>=size && !ascii)
       {
       past_eof:
-	 debug((10,"copy src: seek past eof (seek_pos=%ld, size=%ld)\n",seek_pos,size));
+	 debug((10,_("copy src: seek past eof (seek_pos=%lld, size=%lld)\n"),
+		  (long long)seek_pos,(long long)size));
 	 pos=seek_pos;
 	 eof=true;
 	 return;
@@ -884,8 +890,11 @@ void FileCopyPeerFA::OpenSession()
       }
    }
    session->Open(file,FAmode,seek_pos);
+   session->SetFileURL(orig_url);
    if(mode==PUT)
    {
+      session->SetTryTime(try_time);
+      session->SetRetries(retries);
       if(e_size!=NO_SIZE && e_size!=NO_SIZE_YET)
 	 session->SetSize(e_size);
       if(date!=NO_DATE && date!=NO_DATE_YET)
@@ -925,7 +934,7 @@ int FileCopyPeerFA::Get_LL(int len)
    if(eof)  // OpenSession can set eof=true.
       return 0;
 
-   long io_at=pos;
+   off_t io_at=pos;
    if(GetRealPos()!=io_at) // GetRealPos can alter pos.
       return 0;
 
@@ -936,12 +945,90 @@ int FileCopyPeerFA::Get_LL(int len)
    {
       if(res==FA::DO_AGAIN)
 	 return 0;
+      if(res==FA::FILE_MOVED)
+      {
+	 // handle redirection.
+	 assert(!fxp);
+	 const char *loc_c=session->GetNewLocation();
+	 int max_redirections=max_redir.Query(0);
+	 if(loc_c && loc_c[0] && max_redirections>0)
+	 {
+	    Log::global->Format(3,_("copy: received redirection to `%s'\n"),loc_c);
+	    if(++redirections>max_redirections)
+	    {
+	       SetError(_("Too many redirections"));
+	       return -1;
+	    }
+
+	    char *loc=alloca_strdup(loc_c);
+	    session->Close(); // loc_c is no longer valid.
+
+	    ParsedURL u(loc,true);
+
+	    if(u.proto)
+	    {
+	       if(reuse_later)
+		  SessionPool::Reuse(session);
+
+	       session=FileAccess::New(&u);
+	       reuse_later=true;
+
+	       xfree(file);
+	       file=xstrdup(u.path?u.path:"");
+	       size=NO_SIZE_YET;
+	       date=NO_DATE_YET;
+
+	       xfree(orig_url);
+	       orig_url=xstrdup(loc);
+	    }
+	    else // !proto
+	    {
+	       if(orig_url)
+	       {
+		  int p_ind=url::path_index(orig_url);
+		  char *s=strrchr(orig_url,'/');
+		  int s_ind=s?s-orig_url:-1;
+		  if(p_ind==-1 || s_ind==-1 || s_ind<p_ind)
+		     s_ind=p_ind=strlen(orig_url);
+		  if(loc[0]=='/')
+		  {
+		     orig_url=(char*)xrealloc(orig_url,p_ind+strlen(loc)+1);
+		     strcpy(orig_url+p_ind,loc);
+		  }
+		  else
+		  {
+		     orig_url=(char*)xrealloc(orig_url,s_ind+1+strlen(loc)+1);
+		     strcpy(orig_url+s_ind,"/");
+		     strcpy(orig_url+s_ind+1,loc);
+		  }
+	       }
+
+	       url::decode_string(loc);
+	       char *slash=strrchr(file,'/');
+	       char *new_file;
+	       if(loc[0]!='/' && slash)
+	       {
+		  *slash=0;
+		  new_file=xstrdup(dir_file(file,loc));
+	       }
+	       else
+	       {
+		  new_file=xstrdup(loc);
+	       }
+	       xfree(file);
+	       file=new_file;
+	    }
+	    try_time=0;
+	    retries=0;
+	    current->Timeout(0); // retry with new location.
+	    return 0;
+	 }
+      }
       SetError(session->StrError(res));
       return -1;
    }
    if(res==0)
    {
-      debug((10,"copy src: Read returned 0\n"));
       eof=true;
       LsCache::Add(session,file,FAmode,this);
    }
@@ -953,7 +1040,7 @@ int FileCopyPeerFA::Put_LL(const char *buf,int len)
    if(session->IsClosed())
       OpenSession();
 
-   long io_at=pos;
+   off_t io_at=pos;
    if(GetRealPos()!=io_at) // GetRealPos can alter pos.
       return 0;
 
@@ -967,6 +1054,8 @@ int FileCopyPeerFA::Put_LL(const char *buf,int len)
 	 return 0;
       if(res==FA::STORE_FAILED)
       {
+	 try_time=session->GetTryTime();
+	 retries=session->GetRetries();
 	 session->Close();
 	 if(can_seek && seek_pos>0)
 	    Seek(FILE_END);
@@ -981,7 +1070,7 @@ int FileCopyPeerFA::Put_LL(const char *buf,int len)
    return res;
 }
 
-long FileCopyPeerFA::GetRealPos()
+off_t FileCopyPeerFA::GetRealPos()
 {
    if(session->OpenMode()!=FAmode || fxp)
       return pos;
@@ -1012,14 +1101,27 @@ long FileCopyPeerFA::GetRealPos()
    return pos;
 }
 
+void FileCopyPeerFA::Init()
+{
+   FAmode=FA::RETRIEVE;
+   file=0;
+   session=0;
+   reuse_later=false;
+   orig_url=0;
+   fxp=false;
+   try_time=0;
+   retries=0;
+   redirections=0;
+}
+
 FileCopyPeerFA::FileCopyPeerFA(FileAccess *s,const char *f,int m)
    : FileCopyPeer(m==FA::STORE ? PUT : GET)
 {
+   Init();
    FAmode=m;
    file=xstrdup(f);
    session=s;
    reuse_later=true;
-   fxp=false;
    if(FAmode==FA::LIST || FAmode==FA::LONG_LIST)
       Save(LsCache::SizeLimit());
 }
@@ -1032,16 +1134,19 @@ FileCopyPeerFA::~FileCopyPeerFA()
 	 SessionPool::Reuse(session);
    }
    xfree(file);
+   xfree(orig_url);
 }
 
 FileCopyPeerFA::FileCopyPeerFA(ParsedURL *u,int m)
    : FileCopyPeer(m==FA::STORE ? PUT : GET)
 {
+   Init();
    FAmode=m;
    file=xstrdup(u->path);
    session=FileAccess::New(u);
    reuse_later=true;
-   fxp=false;
+   orig_url=u->orig_url;
+   u->orig_url=0;
    if(!file)
    {
       SetError(_("file name missed in URL"));
@@ -1093,21 +1198,73 @@ FileCopyPeerFDStream::~FileCopyPeerFDStream()
    if(stream && delete_stream)
       delete stream;
 }
+
+void FileCopyPeerFDStream::Seek_LL()
+{
+   int fd=stream->fd;
+   assert(fd!=-1);
+   if(CanSeek(seek_pos))
+   {
+      if(seek_pos==FILE_END)
+      {
+	 seek_pos=lseek(fd,0,SEEK_END);
+	 if(seek_pos==-1)
+	 {
+	    can_seek=false;
+	    can_seek0=false;
+	    seek_pos=0;
+	 }
+	 else
+	 {
+	    SetSize(seek_pos);
+	    if(seek_pos>seek_base)
+	       seek_pos-=seek_base;
+	    else
+	       seek_pos=0;
+	 }
+	 pos=seek_pos;
+      }
+      else
+      {
+	 if(lseek(fd,seek_pos+seek_base,SEEK_SET)==-1)
+	 {
+	    can_seek=false;
+	    can_seek0=false;
+	    seek_pos=0;
+	 }
+	 pos=seek_pos;
+      }
+      if(mode==PUT)
+	 pos+=in_buffer;
+   }
+   else
+   {
+      seek_pos=pos;
+   }
+}
+
 int FileCopyPeerFDStream::getfd()
 {
+   if(stream->fd!=-1)
+      return stream->fd;
    int fd=stream->getfd();
    if(fd==-1)
    {
       if(stream->error())
       {
 	 SetError(stream->error_text);
-	 Timeout(0);
+	 current->Timeout(0);
       }
       else
       {
-	 TimeoutS(1);
+	 current->TimeoutS(1);
       }
+      return -1;
    }
+   pos=0;
+   if(mode==PUT)
+      pos+=in_buffer;
+   Seek_LL();
    return fd;
 }
 int FileCopyPeerFDStream::Do()
@@ -1169,7 +1326,7 @@ int FileCopyPeerFDStream::Do()
 
 bool FileCopyPeerFDStream::IOReady()
 {
-   return seek_pos==0 || stream->fd!=-1;
+   return seek_pos==pos || stream->fd!=-1;
 }
 
 bool FileCopyPeerFDStream::Done()
@@ -1181,7 +1338,7 @@ bool FileCopyPeerFDStream::Done()
    return true;
 }
 
-void FileCopyPeerFDStream::Seek(long new_pos)
+void FileCopyPeerFDStream::Seek(off_t new_pos)
 {
    if(pos==new_pos)
       return;
@@ -1196,34 +1353,33 @@ void FileCopyPeerFDStream::Seek(long new_pos)
    }
 #endif
    super::Seek(new_pos);
-   if(getfd()==-1)
-      return;
-   if(new_pos==FILE_END)
+   int fd=stream->fd;
+   if(fd==-1)
    {
-      new_pos=lseek(getfd(),0,SEEK_END);
-      if(new_pos==-1)
+      if(seek_pos!=FILE_END)
       {
-	 can_seek=false;
-	 can_seek0=false;
+	 pos=seek_pos+(mode==PUT)*in_buffer;
 	 return;
       }
-      SetSize(new_pos);
-      if(new_pos>seek_base)
-	 new_pos-=seek_base;
       else
-	 new_pos=0;
-      pos=new_pos;
-   }
-   else
-   {
-      if(lseek(getfd(),new_pos+seek_base,SEEK_SET)==-1)
       {
-	 can_seek=false;
-	 can_seek0=false;
-	 return;
+	 off_t s=stream->get_size();
+	 if(s!=-1)
+	 {
+	    SetSize(s);
+	    pos=seek_pos+(mode==PUT)*in_buffer;
+	    return;
+	 }
+	 else
+	 {
+	    // ok, have to try getfd.
+	    fd=getfd();
+	 }
       }
-      pos=new_pos;
+      if(fd==-1)
+	 return;
    }
+   Seek_LL();
 }
 
 int FileCopyPeerFDStream::Get_LL(int len)
@@ -1402,11 +1558,12 @@ FileCopyPeerString::FileCopyPeerString(const char *s, int len)
 FileCopyPeerString::~FileCopyPeerString()
 {
 }
-void FileCopyPeerString::Seek(long new_pos)
+void FileCopyPeerString::Seek(off_t new_pos)
 {
    assert(new_pos!=FILE_END);
    UnSkip(pos-new_pos);
    super::Seek(new_pos);
+   pos=new_pos;
 }
 
 // FileCopyPeerDirList
