@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.h,v 1.8 2000/04/15 18:57:29 lav Exp $ */
+/* $Id: FileSet.h,v 1.10 2000/07/17 22:53:39 lav Exp $ */
 
 #ifndef FILESET_H
 #define FILESET_H
@@ -40,7 +40,7 @@ public:
    char	    *name;
    mode_t   mode;
    time_t   date;
-   long	    size;
+   off_t    size;
    void	    *data;
 
    enum	 type
@@ -70,7 +70,8 @@ public:
       }
    FileInfo(const FileInfo &fi);
 
-   void SetName(const char *n) { xfree(name); name=xstrdup(n); defined|=NAME; }
+   void SetName(const char *n);
+   FileInfo(const char *n) { Init(); SetName(n); }
    void SetMode(mode_t m) { mode=m; defined|=MODE; }
    void SetDate(time_t t) { date=t; defined|=DATE; defined&=~DATE_UNPREC; }
    void SetDateUnprec(time_t t)
@@ -82,7 +83,7 @@ public:
    void SetType(type t) { filetype=t; defined|=TYPE; }
    void SetSymlink(const char *s) { xfree(symlink); symlink=xstrdup(s);
       filetype=SYMLINK; defined|=TYPE|SYMLINK_DEF; }
-   void	SetSize(long s) { size=s; defined|=SIZE; }
+   void	SetSize(off_t s) { size=s; defined|=SIZE; }
 
    void	 Merge(const FileInfo&);
 
@@ -96,6 +97,8 @@ public:
 	 data=xmemdup(d,len);
       }
    void  *GetAssociatedData() { return data; }
+
+   operator const char *() { return name; }
 };
 
 class FileSet
@@ -116,8 +119,9 @@ public:
    }
    FileSet(const FileSet *s);
    ~FileSet();
+   void Empty();
 
-   int	 get_fnum() { return fnum; }
+   int	 get_fnum() const { return fnum; }
 
    void	 Add(FileInfo *);
    void	 Merge(const FileSet *);
@@ -143,7 +147,7 @@ public:
 
    FileInfo *FindByName(const char *name) const;
 
-   void  SetSize(const char *name,long size)
+   void  SetSize(const char *name,off_t size)
    {
       FileInfo *f=FindByName(name);
       if(f)
@@ -157,6 +161,8 @@ public:
    }
 
    void SortByName();
+
+   FileInfo * operator[](int i) const { return i<fnum?files[i]:0; }
 };
 
 #endif // FILESET_H
