@@ -69,17 +69,6 @@ FDStream::~FDStream()
    xfree(error_text);
 };
 
-long FDStream::getsize_and_seek_end()
-{
-   int fd=getfd();
-   if(fd==-1)
-      return -1;
-   long size=lseek(fd,0,SEEK_END);
-   if(size<0)
-      size=0;
-   return size;
-}
-
 void OutputFilter::Parent(int *p)
 {
    close(p[0]);
@@ -142,7 +131,7 @@ int OutputFilter::getfd()
       if(second_fd==-1)
       {
 	 if(second->error())
-	    error_text=xstrdup("chain output error");
+	    error_text=xstrdup(_("chain output error"));
 	 return -1;
       }
       if(pg==0)
@@ -157,7 +146,7 @@ int OutputFilter::getfd()
    {
       if(errno==EMFILE || errno==ENFILE)
 	 return -1;
-      sprintf(s,"pipe() failed: %s",strerror(errno));
+      sprintf(s,_("pipe() failed: %s"),strerror(errno));
       error_text=xstrdup(s);
       return -1;
    }
@@ -172,11 +161,13 @@ int OutputFilter::getfd()
       kill(getpid(),SIGSTOP);
       SignalHook::RestoreAll();
       Child(p);
+      if(stderr_to_stdout)
+	 dup2(1,2);
       if(oldcwd)
       {
 	 if(chdir(oldcwd)==-1)
 	 {
-	    fprintf(stderr,"chdir(%s) failed: %s\n",oldcwd,strerror(errno));
+	    fprintf(stderr,_("chdir(%s) failed: %s\n"),oldcwd,strerror(errno));
 	    fflush(stderr);
 	    _exit(1);
 	 }
@@ -184,7 +175,7 @@ int OutputFilter::getfd()
       if(a)
 	 execvp(a->a0(),a->GetV());
       execl("/bin/sh","sh","-c",name,NULL);
-      fprintf(stderr,"execl(/bin/sh) failed: %s\n",strerror(errno));
+      fprintf(stderr,_("execl(/bin/sh) failed: %s\n"),strerror(errno));
       fflush(stderr);
       _exit(1);
    case(-1): /* error */
@@ -221,6 +212,7 @@ void OutputFilter::Init()
    oldcwd=xgetcwd();
    pg=0;
    closed=false;
+   stderr_to_stdout=false;
 }
 
 void OutputFilter::SetCwd(const char *cwd)
@@ -360,3 +352,15 @@ bool FileStream::can_seek()
       return false;  // whatever we seek, the writes will go to end of file.
    return true;
 }
+
+off_t FileStream::get_size()
+{
+   struct stat st;
+   if(-1==(fd==-1?stat(full_name,&st):fstat(fd,&st)))
+   {
+      if(errno==ENOENT)
+	 return 0;   // assume non-existent files to be empty.
+      return -1;
+   }
+   return st.st_size;
+}
