@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.65.2.2 2000/06/20 06:38:49 lav Exp $ */
+/* $Id: Http.cc,v 1.108 2000/09/08 10:53:03 lav Exp $ */
 
 #include <config.h>
 #include <unistd.h>
@@ -29,11 +29,16 @@
 #include <errno.h>
 #include <stdarg.h>
 #include <time.h>
+#include <fnmatch.h>
 #include "Http.h"
 #include "ResMgr.h"
 #include "log.h"
 #include "url.h"
 #include "HttpDir.h"
+#include "misc.h"
+#ifdef USE_SSL
+# include "lftp_ssl.h"
+#endif
 
 #include "ascii_ctype.h"
 
@@ -47,6 +52,7 @@ CDECL char *strptime(const char *buf, const char *format, struct tm *tm);
 
 #define HTTP_DEFAULT_PORT	 "80"
 #define HTTP_DEFAULT_PROXY_PORT	 "3128"
+#define HTTPS_DEFAULT_PORT	 "443"
 
 static time_t http_atotm (const char *time_string);
 static int  base64_length (int len);
@@ -56,6 +62,7 @@ static void base64_encode (const char *s, char *store, int length);
 #define H_20X(x)        (((x) >= 200) && ((x) < 300))
 #define H_PARTIAL(x)    ((x) == 206)
 #define H_REDIRECTED(x) (((x) == 301) || ((x) == 302))
+#define H_EMPTY(x)	(((x) == 204) || ((x) == 205))
 
 
 void Http::Init()
@@ -64,6 +71,7 @@ void Http::Init()
    sock=-1;
    send_buf=0;
    recv_buf=0;
+   recv_buf_suspended=false;
    body_size=-1;
    bytes_received=0;
    line=0;
@@ -71,13 +79,13 @@ void Http::Init()
    status_code=0;
    status_consumed=0;
    proto_version=0x10;
-   location=0;
    sent_eot=false;
+   last_method=0;
 
    default_cwd="/";
 
    keep_alive=false;
-   keep_alive_max=1;
+   keep_alive_max=-1;
 
    array_send=0;
 
@@ -92,9 +100,12 @@ void Http::Init()
    no_cache=false;
 
    hftp=false;
+   https=false;
    use_head=true;
 
    user_agent=0;
+   post_data=0;
+   post=false;
 }
 
 Http::Http() : super()
@@ -118,13 +129,19 @@ Http::Http(const Http *f) : super(f)
 
 Http::~Http()
 {
-   Delete(send_buf);
-   Delete(recv_buf);
-   if(sock!=-1)
-      close(sock);
-   xfree(line);
-   xfree(status);
-   xfree(location);
+   Close();
+   Disconnect();
+}
+
+void Http::MoveConnectionHere(Http *o)
+{
+   send_buf=o->send_buf; o->send_buf=0;
+   recv_buf=o->recv_buf; o->recv_buf=0;
+   sock=o->sock; o->sock=-1;
+   rate_limit=o->rate_limit; o->rate_limit=0;
+   last_method=o->last_method; o->last_method=0;
+   state=CONNECTED;
+   o->Disconnect();
 }
 
 void Http::Disconnect()
@@ -140,38 +157,85 @@ void Http::Disconnect()
    }
    if(sock!=-1)
    {
+      DebugPrint("---- ",_("Closing HTTP connection"),7);
       close(sock);
       sock=-1;
    }
+   if((mode==STORE && state!=DONE && real_pos>0)
+   && !Error())
+   {
+      if(last_method && !strcmp(last_method,"POST"))
+	 SetError(FATAL,_("POST method failed"));
+      else
+	 SetError(STORE_FAILED,0);
+   }
+   last_method=0;
+   ResetRequestData();
+   state=DISCONNECTED;
+}
+
+void Http::ResetRequestData()
+{
    body_size=-1;
    bytes_received=0;
-   real_pos=-1;
+   real_pos=no_ranges?0:-1;
    xfree(status);
    status=0;
    status_consumed=0;
-   xfree(location);
-   location=0;
+   xfree(line);
+   line=0;
    sent_eot=false;
    keep_alive=false;
-   keep_alive_max=1;
+   keep_alive_max=-1;
    array_send=array_ptr;
    chunked=false;
    chunk_size=-1;
    chunk_pos=0;
    seen_ranges_bytes=false;
-   if(mode==STORE && state!=DONE && !Error())
-      SetError(STORE_FAILED,0);
-   else
-      state=DISCONNECTED;
 }
 
 void Http::Close()
 {
-   Disconnect();
+   if(mode==CLOSED)
+      return;
+   if(recv_buf)
+      recv_buf->Do();	// try to read any remaining data
+   if(sock!=-1 && keep_alive && (keep_alive_max>1 || keep_alive_max==-1)
+   && mode!=STORE && !recv_buf->Eof() && (state==RECEIVING_BODY || state==DONE))
+   {
+      recv_buf->Resume();
+      Roll(recv_buf);
+      if(xstrcmp(last_method,"HEAD"))
+      {
+	 // check if all data are in buffer
+	 if(!chunked)	// chunked is a bit complex, so don't handle it
+	 {
+	    bytes_received+=recv_buf->Size();
+	    recv_buf->Skip(recv_buf->Size());
+	 }
+	 if(!(body_size>=0 && bytes_received==body_size))
+	    goto disconnect;
+      }
+      // can reuse the connection.
+      state=CONNECTED;
+      ResetRequestData();
+      idle_start=now;
+      TimeoutS(idle);
+      delete rate_limit;
+      rate_limit=0;
+   }
+   else
+   {
+   disconnect:
+      try_time=0;
+      Disconnect();
+   }
    array_send=0;
    no_cache_this=false;
    no_ranges=false;
-   bytes_received=0;
+   post=false;
+   xfree(post_data);
+   post_data=0;
    super::Close();
 }
 
@@ -183,7 +247,6 @@ void Http::Send(const char *format,...)
 {
    va_list va;
    va_start(va,format);
-
    char *str;
 #ifdef HAVE_VSNPRINTF
    static int max_send=256;
@@ -203,10 +266,8 @@ void Http::Send(const char *format,...)
    str=string_alloca(2048);
    vsprintf(str,format,va);
 #endif
-
    DebugPrint("---> ",str,5);
    send_buf->Put(str);
-
    va_end(va);
 }
 
@@ -216,11 +277,40 @@ void Http::SendMethod(const char *method,const char *efile)
    url::encode_string(hostname,ehost);
    if(!use_head && !strcmp(method,"HEAD"))
       method="GET";
+   last_method=method;
+   if(file_url)
+   {
+      efile=file_url;
+      if(!proxy)
+	 efile+=url::path_index(efile);
+      else if(!strncmp(efile,"hftp://",7))
+	 efile++;
+   }
+
+   if(hftp && mode!=LONG_LIST && mode!=CHANGE_DIR && mode!=MAKE_DIR
+   && (strlen(efile)<7 || strncmp(efile+strlen(efile)-7,";type=",6)))
+   {
+      char *pfile=alloca_strdup2(efile,7);
+      sprintf(pfile,"%s;type=%c",efile,ascii?'a':'i');
+      efile=pfile;
+   }
+
    Send("%s %s HTTP/1.1\r\n",method,efile);
    Send("Host: %s\r\n",ehost);
    if(user_agent && user_agent[0])
       Send("User-Agent: %s\r\n",user_agent);
-   Send("Accept: */*\r\n");
+   if(!hftp)
+   {
+      const char *accept=Query("accept");
+      if(accept && accept[0])
+	 Send("Accept: %s\r\n",accept);
+      accept=Query("accept-language");
+      if(accept && accept[0])
+	 Send("Accept-Language: %s\r\n",accept);
+      accept=Query("accept-charset");
+      if(accept && accept[0])
+	 Send("Accept-Charset: %s\r\n",accept);
+   }
 }
 
 
@@ -281,6 +371,8 @@ void Http::SendRequest(const char *connection,const char *f)
    if(proxy)
    {
       const char *proto="http";
+      if(https)
+	 proto="https";
       if(hftp)
       {
 	 if(user && pass)
@@ -330,14 +422,6 @@ add_path:
       memcpy(path_base+1,"%2F",3);
    }
 
-   if(hftp && mode!=LONG_LIST && mode!=CHANGE_DIR && mode!=MAKE_DIR)
-   {
-      if(ascii)
-	 strcat(pfile,";type=a");
-      else
-	 strcat(pfile,";type=i");
-   }
-
    efile=pfile;
    efile_len=strlen(efile);
 
@@ -351,6 +435,11 @@ add_path:
    case CLOSED:
    case CONNECT_VERIFY:
    case QUOTE_CMD:
+      if(post)
+      {
+	 entity_size=xstrlen(post_data);
+	 goto send_post;
+      }
    case RENAME:
    case LIST:
    case CHANGE_MODE:
@@ -360,27 +449,31 @@ add_path:
    retrieve:
       SendMethod("GET",efile);
       if(pos>0 && !no_ranges)
-	 Send("Range: bytes=%ld-\r\n",pos);
+	 Send("Range: bytes=%lld-\r\n",(long long)pos);
       break;
 
    case STORE:
-      SendMethod("PUT",efile);
+      if(hftp || strcasecmp(Query("put-method",hostname),"POST"))
+	 SendMethod("PUT",efile);
+      else
+      {
+      send_post:
+	 SendMethod("POST",efile);
+	 pos=0;
+      }
       if(entity_size>=0)
-	 Send("Content-length: %ld\r\n",entity_size-pos);
+	 Send("Content-length: %lld\r\n",(long long)(entity_size-pos));
       if(pos>0 && entity_size<0)
-	 Send("Range: bytes=%ld-\r\n",pos);
+	 Send("Range: bytes=%lld-\r\n",(long long)pos);
       else if(pos>0)
-	 Send("Range: bytes=%ld-%ld/%ld\r\n",pos,entity_size-1,entity_size);
+	 Send("Range: bytes=%lld-%lld/%lld\r\n",(long long)pos,
+		     (long long)(entity_size-1),(long long)entity_size);
       if(entity_date!=(time_t)-1)
       {
 	 char d[256];
 	 static const char weekday_names[][4]={
 	    "Sun","Mon","Tue","Wed","Thu","Fri","Sat"
 	 };
-	 static const char month_names[][4]={
-	    "Jan","Feb","Mar","Apr","May","Jun",
-	    "Jul","Aug","Sep","Oct","Nov","Dec",
-	 };
 	 struct tm *t=gmtime(&entity_date);
 	 sprintf(d,"%s, %2d %s %04d %02d:%02d:%02d GMT",
 	    weekday_names[t->tm_wday],t->tm_mday,month_names[t->tm_mon],
@@ -419,15 +512,32 @@ add_path:
    }
    if(!hftp)
    {
-      const char *cookie=Query("cookie",hostname);
+      char *cookie=MakeCookie(hostname,efile+(proxy?url::path_index(efile):0));
       if(cookie && cookie[0])
 	 Send("Cookie: %s\r\n",cookie);
+      xfree(cookie);
+
+      const char *content_type=0;
+      if(!strcmp(last_method,"PUT"))
+	 content_type=Query("put-content-type",hostname);
+      else if(!strcmp(last_method,"POST"))
+	 content_type=Query("post-content-type",hostname);
+      if(content_type && content_type[0])
+	 Send("Content-Type: %s\r\n",content_type);
    }
    if(mode==ARRAY_INFO && !use_head)
       connection="close";
+   else if(mode!=STORE)
+      connection="keep-alive";
    if(mode!=ARRAY_INFO || connection)
       Send("Connection: %s\r\n",connection?connection:"close");
    Send("\r\n");
+   if(post)
+   {
+      if(post_data)
+	 Send("%s",post_data);
+      entity_size=NO_SIZE;
+   }
 
    keep_alive=false;
    chunked=false;
@@ -438,10 +548,16 @@ add_path:
 
 void Http::SendArrayInfoRequest()
 {
-   while(array_send-array_ptr<keep_alive_max
-	 && array_send<array_cnt)
+   int m=1;
+   if(keep_alive)
    {
-      SendRequest(array_send==array_cnt-1 ? "close" : "keep-alive",
+      m=keep_alive_max;
+      if(m==-1)
+	 m=100;
+   }
+   while(array_send-array_ptr<m && array_send<array_cnt)
+   {
+      SendRequest(array_send==array_cnt-1 ? 0 : "keep-alive",
 	 array_for_info[array_send].file);
       array_send++;
    }
@@ -452,10 +568,13 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 {
    if(!strcasecmp(name,"Content-length"))
    {
-      sscanf(value,"%ld",&body_size);
+      long long bs=0;
+      if(1!=sscanf(value,"%lld",&bs))
+	 return;
+      body_size=bs;
       if(pos==0 && mode!=STORE)
 	 entity_size=body_size;
-      if(pos==0 && opt_size)
+      if(pos==0 && opt_size && H_20X(status_code))
 	 *opt_size=body_size;
 
       if(mode==ARRAY_INFO && H_20X(status_code))
@@ -468,21 +587,21 @@ void Http::HandleHeaderLine(const char *name,const char *value)
    }
    if(!strcasecmp(name,"Content-range"))
    {
-      long first,last,fsize;
-      if(sscanf(value,"%*s %ld-%ld/%ld",&first,&last,&fsize)!=3)
+      long long first,last,fsize;
+      if(sscanf(value,"%*s %lld-%lld/%lld",&first,&last,&fsize)!=3)
 	 return;
       real_pos=first;
       body_size=last-first+1;
       if(mode!=STORE)
 	 entity_size=fsize;
-      if(opt_size)
+      if(opt_size && H_20X(status_code))
 	 *opt_size=fsize;
       return;
    }
    if(!strcasecmp(name,"Last-Modified"))
    {
       time_t t=http_atotm(value);
-      if(opt_date)
+      if(opt_date && H_20X(status_code))
 	 *opt_date=t;
 
       if(mode==ARRAY_INFO && H_20X(status_code))
@@ -505,6 +624,8 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       const char *m=strstr(value,"max=");
       if(m)
 	 sscanf(m+4,"%d",&keep_alive_max);
+      else
+	 keep_alive_max=100;
       return;
    }
    if(!strcasecmp(name,"Connection")
@@ -534,6 +655,9 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 	 seen_ranges_bytes=true;
       return;
    }
+   if(!strcasecmp(name,"Set-Cookie")
+   && !hftp && (bool)Query("set-cookies",hostname))
+      SetCookie(value);
 }
 
 static const char *find_eol(const char *str,int len)
@@ -549,6 +673,48 @@ static const char *find_eol(const char *str,int len)
    return 0;
 }
 
+void Http::GetBetterConnection(int level,int count)
+{
+   if(level==0)
+      return;
+   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
+   {
+      Http *o=(Http*)fo; // we are sure it is Http.
+
+      if(o->sock==-1 || o->state==CONNECTING)
+	 continue;
+
+      if(o->state!=CONNECTED || o->mode!=CLOSED)
+      {
+	 if(level<2)
+	    continue;
+	 if(!connection_takeover || o->priority>=priority)
+	    continue;
+	 o->Disconnect();
+	 return;
+      }
+      else
+      {
+	 takeover_time=now;
+      }
+
+      // connected session (o) must have resolved address
+      if(!peer)
+      {
+	 // copy resolved address so that it would be possible to create
+	 // data connection.
+	 xfree(peer);
+	 peer=(sockaddr_u*)xmemdup(o->peer,o->peer_num*sizeof(*o->peer));
+	 peer_num=o->peer_num;
+	 peer_curr=o->peer_curr;
+      }
+
+      // so borrow the connection
+      MoveConnectionHere(o);
+      return;
+   }
+}
+
 int Http::Do()
 {
    int m=STALL;
@@ -556,9 +722,9 @@ int Http::Do()
    const char *buf;
    int len;
    Buffer *data_buf;
+   int count;
 
    // check if idle time exceeded
-   // NOT USED YET.
    if(mode==CLOSED && sock!=-1 && idle>0)
    {
       if(now-idle_start>=idle)
@@ -575,14 +741,64 @@ int Http::Do()
    ExpandTildeInCWD();
 
    if(Error())
-      return MOVED;
+      return m;
 
    switch(state)
    {
    case DISCONNECTED:
       if(mode==CLOSED || !hostname)
 	 return m;
-      if(!ModeSupported())
+      if(!hftp && mode==QUOTE_CMD && !post)
+      {
+      handle_quote_cmd:
+	 if(file && !strncasecmp(file,"Set-Cookie ",11))
+	    SetCookie(file+11);
+	 else if(file && !strncasecmp(file,"POST ",5))
+	 {
+	    // POST encoded_path data
+	    const char *scan=file+5;
+	    while(*scan==' ')
+	       scan++;
+	    char *url=string_alloca(5+xstrlen(hostname)*3+1+xstrlen(portname)*3
+				    +1+xstrlen(cwd)*3+1+strlen(scan)+1);
+	    strcpy(url,https?"https://":"http://");
+	    url::encode_string(hostname,url+strlen(url),URL_HOST_UNSAFE);
+	    if(portname)
+	    {
+	       strcat(url,":");
+	       url::encode_string(portname,url+strlen(url),URL_PORT_UNSAFE);
+	    }
+	    if(*scan!='/' && cwd)
+	    {
+	       if(cwd[0]!='/')
+		  strcat(url,"/");
+	       url::encode_string(cwd,url+strlen(url),URL_PATH_UNSAFE);
+	    }
+	    if(*scan!='/')
+	       strcat(url,"/");
+	    strcat(url,scan);
+	    char *path=xstrdup(url);
+	    char *space=strchr(path,' ');
+	    if(space)
+	       *space=0;
+	    scan=strchr(scan,' ');
+	    while(scan && *scan==' ')
+	       scan++;
+	    post=true;
+	    post_data=xstrdup(scan);
+	    xfree(file_url);
+	    file_url=path;
+	    return MOVED;
+	 }
+	 else
+	 {
+	    SetError(NOT_SUPP,0);
+	    return MOVED;
+	 }
+	 state=DONE;
+	 return MOVED;
+      }
+      if(!post && !ModeSupported())
       {
 	 SetError(NOT_SUPP);
 	 return MOVED;
@@ -592,16 +808,30 @@ int Http::Do()
 	 if(!proxy)
 	 {
 	    // problem here: hftp cannot work without proxy
-	    SetError(FATAL,"ftp over http cannot work without proxy, set hftp:proxy.");
+	    SetError(FATAL,_("ftp over http cannot work without proxy, set hftp:proxy."));
 	    return MOVED;
 	 }
       }
 
+      // walk through Http classes and try to find identical idle session
+      // first try "easy" cases of session take-over.
+      count=CountConnections();
+      for(int i=0; i<3; i++)
+      {
+	 if(i>=2 && (connection_limit==0 || connection_limit>count))
+	    break;
+	 GetBetterConnection(i,count);
+	 if(state!=DISCONNECTED)
+	    return MOVED;
+      }
+
       if(!ReconnectAllowed())
 	 return m;
 
-      if(Resolve(HTTP_DEFAULT_PORT,"http","tcp")==MOVED)
-	 m=MOVED;
+      if(https)
+	 m|=Resolve(HTTPS_DEFAULT_PORT,"https","tcp");
+      else
+	 m|=Resolve(HTTP_DEFAULT_PORT,"http","tcp");
       if(!peer)
 	 return m;
 
@@ -627,7 +857,7 @@ int Http::Do()
 	    return m;
 	 }
 	 char str[256];
-	 sprintf(str,"cannot create socket of address family %d",
+	 sprintf(str,_("cannot create socket of address family %d"),
 			peer[peer_curr].sa.sa_family);
 	 SetError(SEE_ERRNO,str);
 	 return MOVED;
@@ -647,7 +877,7 @@ int Http::Do()
       )
       {
 	 NextPeer();
-	 Log::global->Format(0,"connect: %s",strerror(errno));
+	 Log::global->Format(0,_("connect: %s"),strerror(errno));
 	 Disconnect();
 	 if(NotSerious(errno))
 	    return MOVED;
@@ -677,10 +907,44 @@ int Http::Do()
       }
 
       m=MOVED;
-      send_buf=new FileOutputBuffer(new FDStream(sock,"<output-socket>"));
-      recv_buf=new FileInputBuffer(new FDStream(sock,"<input-socket>"));
-
-      DebugPrint("---- ","Sending request...");
+      state=CONNECTED;
+#ifdef USE_SSL
+      if(proxy?!strncmp(proxy,"https://",8):https)
+      {
+	 SSL *ssl=lftp_ssl_new(sock);
+	 IOBufferSSL *send_buf_ssl=new IOBufferSSL(ssl,IOBuffer::PUT);
+	 IOBufferSSL *recv_buf_ssl=new IOBufferSSL(ssl,IOBuffer::GET);
+	 send_buf_ssl->DoConnect();
+	 recv_buf_ssl->CloseLater();
+	 send_buf=send_buf_ssl;
+	 recv_buf=recv_buf_ssl;
+      }
+      else
+#endif
+      {
+	 send_buf=new IOBufferFDStream(
+	    new FDStream(sock,"<output-socket>"),IOBuffer::PUT);
+	 recv_buf=new IOBufferFDStream(
+	    new FDStream(sock,"<input-socket>"),IOBuffer::GET);
+      }
+      /*fallthrough*/
+   case CONNECTED:
+      if(mode==QUOTE_CMD && !post)
+	 goto handle_quote_cmd;
+      if(recv_buf->Eof())
+      {
+	 DebugPrint("**** ",_("Peer closed connection"),0);
+	 Disconnect();
+	 return MOVED;
+      }
+      if(mode==CLOSED)
+	 return m;
+      if(!post && !ModeSupported())
+      {
+	 SetError(NOT_SUPP);
+	 return MOVED;
+      }
+      DebugPrint("---- ",_("Sending request..."));
       if(mode==ARRAY_INFO)
       {
 	 SendArrayInfoRequest();
@@ -701,7 +965,7 @@ int Http::Do()
    case RECEIVING_HEADER:
       if(send_buf->Error() || recv_buf->Error())
       {
-	 if(mode==STORE && status_code && !H_20X(status_code))
+	 if((mode==STORE || post) && status_code && !H_20X(status_code))
 	    goto pre_RECEIVING_BODY;   // assume error.
 	 if(send_buf->Error())
 	    DebugPrint("**** ",send_buf->ErrorText(),0);
@@ -718,7 +982,10 @@ int Http::Do()
       if(!buf)
       {
 	 // eof
-	 DebugPrint("**** ","Hit EOF while fetching headers",0);
+	 DebugPrint("**** ",_("Hit EOF while fetching headers"),0);
+	 // workaround some broken servers
+	 if(H_REDIRECTED(status_code) && location)
+	    goto pre_RECEIVING_BODY;
 	 Disconnect();
 	 return MOVED;
       }
@@ -743,12 +1010,11 @@ int Http::Do()
 		     array_for_info[array_ptr].size=-1;
 		  if(++array_ptr>=array_cnt)
 		  {
-		     Disconnect();
 		     state=DONE;
 		     return MOVED;
 		  }
-		  // if protocol is HTTP/1.1, we can avoid reconnection
-		  if(proto_version>=0x11 && keep_alive)
+		  // we can avoid reconnection if server supports it.
+		  if(keep_alive && (keep_alive_max>1 || keep_alive_max==-1))
 		  {
 		     SendArrayInfoRequest();
 		  }
@@ -771,7 +1037,7 @@ int Http::Do()
 		  if(!sent_eot && H_20X(status_code))
 		  {
 		     // should never happen
-		     DebugPrint("**** ","Success, but did nothing??",0);
+		     DebugPrint("**** ",_("Success, but did nothing??"),0);
 		     Disconnect();
 		     return MOVED;
 		  }
@@ -805,7 +1071,7 @@ int Http::Do()
 		  status_code=200;
 		  if(1!=sscanf(status,"HTTP %n%d",&status_consumed,&status_code))
 		  {
-		     DebugPrint("**** ","Could not parse HTTP status line",0);
+		     DebugPrint("**** ",_("Could not parse HTTP status line"),0);
 		     //FIXME: STORE
 		     goto pre_RECEIVING_BODY;
 		  }
@@ -813,6 +1079,13 @@ int Http::Do()
 	       proto_version=(ver_major<<4)+ver_minor;
 	       if(!H_20X(status_code))
 	       {
+		  if(status_code==100)
+		  {
+		     // 100 Continue
+		     status_code=0;
+		     xfree(status);
+		  }
+
 		  if(status_code/100==5) // server failed, try another
 		     NextPeer();
 		  // check for retriable codes
@@ -826,10 +1099,7 @@ int Http::Do()
 		  }
 
 		  if(mode==ARRAY_INFO)
-		  {
 		     retries=0;
-		     return MOVED;
-		  }
 
 		  return MOVED;
 	       }
@@ -857,6 +1127,10 @@ int Http::Do()
 
    pre_RECEIVING_BODY:
 
+      // 204 No Content
+      if(H_EMPTY(status_code) && body_size<0)
+	 body_size=0;
+
       if(H_REDIRECTED(status_code))
       {
 	 // check if it is redirection to the same server
@@ -867,10 +1141,14 @@ int Http::Do()
       if(!H_20X(status_code))
       {
 	 char *err=(char*)alloca(strlen(status)+strlen(file)+strlen(cwd)+xstrlen(location)+20);
+	 int code=NO_FILE;
 
 	 if(H_REDIRECTED(status_code))
+	 {
 	    sprintf(err,"%s (%s -> %s)",status+status_consumed,file,
 				    location?location:"nowhere");
+	    code=FILE_MOVED;
+	 }
 	 else
 	 {
 	    if(file && file[0])
@@ -879,20 +1157,19 @@ int Http::Do()
 	       sprintf(err,"%s (%s%s)",status+status_consumed,cwd,
 		  (cwd[0] && cwd[strlen(cwd)-1]=='/')?"":"/");
 	 }
-	 Disconnect();
-	 SetError(NO_FILE,err);
+	 state=RECEIVING_BODY;
+	 SetError(code,err);
 	 return MOVED;
       }
       if(mode==CHANGE_DIR)
       {
 	 xfree(cwd);
 	 cwd=xstrdup(file);
-	 Disconnect();
 	 state=DONE;
 	 return MOVED;
       }
 
-      DebugPrint("---- ","Receiving body...");
+      DebugPrint("---- ",_("Receiving body..."));
       assert(rate_limit==0);
       rate_limit=new RateLimit();
       if(real_pos<0) // assume Range: did not work
@@ -900,7 +1177,7 @@ int Http::Do()
 	 if(mode!=STORE && body_size>=0)
 	 {
 	    entity_size=body_size;
-	    if(opt_size)
+	    if(opt_size && H_20X(status_code))
 	       *opt_size=entity_size;
 	 }
 	 real_pos=0;
@@ -941,7 +1218,11 @@ int Http::Do()
 	    bool no_ranges_if_timeout=(bytes_received==0 && !seen_ranges_bytes);
 	    if(CheckTimeout())
 	    {
-	       no_ranges|=no_ranges_if_timeout;
+	       if(no_ranges_if_timeout)
+	       {
+		  no_ranges=true;
+		  real_pos=0; // so that pget would know immediately.
+	       }
 	       return MOVED;
 	    }
 	 }
@@ -973,6 +1254,33 @@ void  Http::ClassInit()
    // register the class
    Register("http",Http::New);
    Register("hftp",HFtp::New);
+#ifdef USE_SSL
+   Register("https",Https::New);
+#endif
+}
+
+void Http::Suspend()
+{
+   if(suspended)
+      return;
+   if(recv_buf)
+   {
+      recv_buf_suspended=recv_buf->IsSuspended();
+      recv_buf->Suspend();
+   }
+   if(send_buf)
+      send_buf->Suspend();
+   super::Suspend();
+}
+void Http::Resume()
+{
+   if(!suspended)
+      return;
+   super::Resume();
+   if(recv_buf && !recv_buf_suspended)
+      recv_buf->Resume();
+   if(send_buf)
+      send_buf->Resume();
 }
 
 int Http::Read(void *buf,int size)
@@ -996,32 +1304,33 @@ int Http::Read(void *buf,int size)
       recv_buf->Get(&buf1,&size1);
       if(buf1==0) // eof
       {
-	 DebugPrint("---- ","Hit EOF");
+	 DebugPrint("---- ",_("Hit EOF"));
 	 if(bytes_received<body_size || chunked)
 	 {
-	    DebugPrint("**** ","Received not enough data, retrying",0);
+	    DebugPrint("**** ",_("Received not enough data, retrying"),0);
 	    Disconnect();
 	    return DO_AGAIN;
 	 }
 	 return 0;
       }
-      if(size1==0)
-	 return DO_AGAIN;
       if(body_size>=0 && bytes_received>=body_size)
       {
-	 DebugPrint("---- ","Received all");
+	 DebugPrint("---- ",_("Received all"));
 	 return 0; // all received
       }
       if(entity_size>=0 && pos>=entity_size)
       {
-	 DebugPrint("---- ","Received all (total)");
+	 DebugPrint("---- ",_("Received all (total)"));
 	 return 0;
       }
+      if(size1==0)
+	 return DO_AGAIN;
       if(chunked)
       {
+	 const char *nl;
 	 if(chunk_size==-1) // expecting first/next chunk
 	 {
-	    const char *nl=(const char*)memchr(buf1,'\n',size1);
+	    nl=(const char*)memchr(buf1,'\n',size1);
 	    if(nl==0)  // not yet
 	    {
 	    not_yet:
@@ -1042,6 +1351,7 @@ int Http::Read(void *buf,int size)
 	 if(chunk_size==0) // eof
 	 {
 	    // FIXME: headers may follow
+	    // to avoid messing with headers, we close connection.
 	    Disconnect();
 	    state=DONE;
 	    return 0;
@@ -1063,6 +1373,12 @@ int Http::Read(void *buf,int size)
 	 if(size1>chunk_size-chunk_pos)
 	    size1=chunk_size-chunk_pos;
       }
+      else
+      {
+	 // limit by body_size.
+	 if(body_size>=0 && size1+bytes_received>=body_size)
+	    size1=body_size-bytes_received;
+      }
 
       int bytes_allowed=rate_limit->BytesAllowed();
       if(size1>bytes_allowed)
@@ -1073,7 +1389,7 @@ int Http::Read(void *buf,int size)
 	 return DO_AGAIN;
       if(real_pos<pos)
       {
-	 long to_skip=pos-real_pos;
+	 off_t to_skip=pos-real_pos;
 	 if(to_skip>size1)
 	    to_skip=size1;
 	 recv_buf->Skip(to_skip);
@@ -1143,7 +1459,7 @@ int Http::Write(const void *buf,int size)
       {
 	 DebugPrint("**** ",strerror(errno),0);
 	 Disconnect();
-	 return STORE_FAILED;
+	 return error_code;
       }
       SetError(SEE_ERRNO,0);
       Disconnect();
@@ -1197,6 +1513,8 @@ const char *Http::CurrentStatus()
       return "";
    case CONNECTING:
       return(_("Connecting..."));
+   case CONNECTED:
+      return(_("Connection idle"));
    case RECEIVING_HEADER:
       if(mode==STORE && !sent_eot && !status)
 	 return(_("Sending data"));
@@ -1221,7 +1539,23 @@ void Http::Reconfig(const char *name)
    if(!hftp && NoProxy())
       SetProxy(0);
    else
-      SetProxy(Query("proxy",c));
+   {
+      const char *p=0;
+      if(hftp && vproto && !strcmp(vproto,"ftp"))
+      {
+	 p=ResMgr::Query("ftp:proxy",c);
+	 if(p && strncmp(p,"http://",7) && strncmp(p,"https://",8))
+	    p=0;
+      }
+      if(!p)
+      {
+	 if(https)
+	    p=ResMgr::Query("https:proxy",c);
+	 else
+	    p=Query("proxy",c);
+      }
+      SetProxy(p);
+   }
 
    if(sock!=-1)
       SetSocketBuffer(sock,socket_buffer);
@@ -1270,6 +1604,220 @@ ListInfo *Http::MakeListInfo()
    return new HttpListInfo(this);
 }
 
+bool Http::CookieClosureMatch(const char *closure_c,
+			      const char *hostname,const char *efile)
+{
+   if(!closure_c)
+      return true;
+   char *closure=alloca_strdup2(closure_c,1);
+   char *path=0;
+
+   char *scan=closure;
+   for(;;)
+   {
+      char *slash=strchr(scan,';');
+      if(!slash)
+	 break;
+      *slash=0;
+      if(!strncmp(slash+1,"path=",5))
+	 path=slash+1+5;
+      else if(!strncmp(slash+1,"secure",6) && (slash[7]==';' || slash[7]==0))
+      {
+	 if(!https)
+	    return false;
+      }
+   }
+   if(closure[0] && 0!=fnmatch(closure,hostname,FNM_PATHNAME))
+      return false;
+   if(!path)
+      return true;
+   int path_len=strlen(path);
+   if(path_len>0 && path[path_len-1]=='/')
+      path_len--;
+   if(!strncmp(efile,path,path_len)
+   && (efile[path_len]==0 || efile[path_len]=='/'))
+      return true;
+   return false;
+}
+
+void Http::CookieMerge(char **all_p,const char *cookie_c)
+{
+   char *all=*all_p;
+   int all_len=xstrlen(all);
+   all=*all_p=(char*)xrealloc(all,all_len+2+xstrlen(cookie_c)+1);
+   all[all_len]=0;   // in case the buffer is freshly allocated.
+
+   char *value=alloca_strdup(cookie_c);
+
+   for(char *entry=strtok(value,";"); entry; entry=strtok(0,";"))
+   {
+      if(*entry==' ')
+	 entry++;
+      if(*entry==0)
+	 break;
+      if(!strncasecmp(entry,"path=",5)
+      || !strncasecmp(entry,"expires=",8)
+      || !strncasecmp(entry,"domain=",7)
+      || (!strncasecmp(entry,"secure",6)
+	  && (entry[6]==' ' || entry[6]==0 || entry[6]==';')))
+	 continue; // filter out path= expires= domain= secure
+
+      char *c_name=entry;
+      char *c_value=strchr(entry,'=');
+      if(c_value)
+	 *c_value++=0;
+      else
+	 c_value=c_name, c_name=0;
+      int c_name_len=xstrlen(c_name);
+
+      char *scan=all;
+      for(;;)
+      {
+	 while(*scan==' ') scan++;
+	 if(*scan==0)
+	    break;
+
+	 char *semicolon=strchr(scan,';');
+	 char *eq=strchr(scan,'=');
+	 if(semicolon && eq>semicolon)
+	    eq=0;
+	 if((eq==0 && c_name==0)
+	 || (eq-scan==c_name_len && !strncmp(scan,c_name,c_name_len)))
+	 {
+	    // remove old cookie.
+	    const char *m=semicolon?semicolon+1:"";
+	    while(*m==' ') m++;
+	    if(*m==0)
+	    {
+	       while(scan>all && scan[-1]==' ')
+		  scan--;
+	       if(scan>all && scan[-1]==';')
+		  scan--;
+	       *scan=0;
+	    }
+	    else
+	       memmove(scan,m,strlen(m)+1);
+	    break;
+	 }
+	 if(!semicolon)
+	    break;
+	 scan=semicolon+1;
+      }
+
+      // append cookie.
+      int c_len=strlen(all);
+      while(c_len>0 && all[c_len-1]==' ')
+	 all[--c_len]=0;  // trim
+      if(c_len>0 && all[c_len-1]!=';')
+	 all[c_len++]=';', all[c_len++]=' ';  // append '; '
+      if(c_name)
+	 sprintf(all+c_len,"%s=%s",c_name,c_value);
+      else
+	 strcpy(all+c_len,c_value);
+   }
+}
+
+char *Http::MakeCookie(const char *hostname,const char *efile)
+{
+   ResMgr::Resource *scan=0;
+   const char *closure;
+   char *all_cookies=0;
+   for(;;)
+   {
+      const char *cookie=ResMgr::QueryNext("http:cookie",&closure,&scan);
+      if(cookie==0)
+	 break;
+      if(!CookieClosureMatch(closure,hostname,efile))
+	 continue;
+      CookieMerge(&all_cookies,cookie);
+   }
+   return all_cookies;
+}
+
+void Http::SetCookie(const char *value_const)
+{
+   char *value=alloca_strdup(value_const);
+   const char *domain=hostname;
+   const char *path=0;
+   bool secure=false;
+
+   for(char *entry=strtok(value,";"); entry; entry=strtok(0,";"))
+   {
+      while(*entry==' ')   // skip spaces.
+	 entry++;
+      if(*entry==0)
+	 break;
+
+      if(!strncasecmp(entry,"expires=",8))
+	 continue; // not used yet
+
+      if(!strncasecmp(entry,"secure",6)
+      && (entry[6]==' ' || entry[6]==0 || entry[6]==';'))
+      {
+	 secure=true;
+	 continue;
+      }
+
+      if(!strncasecmp(entry,"path=",5))
+      {
+	 path=alloca_strdup(entry+5);
+      	 continue;
+      }
+
+      if(!strncasecmp(entry,"domain=",7))
+      {
+	 char *new_domain=alloca_strdup(entry+6);
+	 if(new_domain[1]=='.')
+	    new_domain[0]='*';
+	 else
+	    new_domain++;
+	 char *end=strchr(new_domain,';');
+	 if(end)
+	    *end=0;
+	 domain=new_domain;
+	 continue;
+      }
+   }
+
+   char *closure=string_alloca(strlen(domain)+xstrlen(path)+32);
+   strcpy(closure,domain);
+   if(path && path[0] && strcmp(path,"/"))
+   {
+      strcat(closure,";path=");
+      strcat(closure,path);
+   }
+   if(secure)
+      strcat(closure,";secure");
+
+   const char *old=Query("cookie",closure);
+
+   char *c=xstrdup(old,2+strlen(value_const));
+   CookieMerge(&c,value_const);
+
+   ResMgr::Set("http:cookie",closure,c);
+
+   xfree(c);
+}
+
+#ifdef USE_SSL
+#undef super
+#define super Http
+Https::Https()
+{
+   https=true;
+   res_prefix="http";
+}
+Https::~Https()
+{
+}
+Https::Https(const Https *o) : super(o)
+{
+   https=true;
+   res_prefix="http";
+   Reconfig(0);
+}
+FileAccess *Https::New(){ return new Https();}
+#endif
 
 #undef super
 #define super Http
@@ -1463,8 +2011,8 @@ base64_encode (const char *s, char *store, int length)
 }
 
 
-
-#ifdef MODULE
+#include "modconfig.h"
+#ifdef MODULE_PROTO_HTTP
 CDECL void module_init()
 {
    Http::ClassInit();
