@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: HttpDir.cc,v 1.44.2.1 2000/06/08 23:08:37 lav Exp $ */
+/* $Id: HttpDir.cc,v 1.56 2000/08/26 19:11:24 lav Exp $ */
 
 #include <config.h>
 #include "ascii_ctype.h"
@@ -215,6 +215,8 @@ static void decode_amps(char *s)
 
 #define debug(str) Log::global->Format(10,"* %s\n",str)
 
+// this procedure is highly inefficient in some cases,
+// esp. when it has to return for more data many times.
 static int parse_html(const char *buf,int len,bool eof,Buffer *list,
       FileSet *set,FileSet *all_links,ParsedURL *prefix,char **base_href,
       LsOptions *lsopt=0)
@@ -233,6 +235,26 @@ static int parse_html(const char *buf,int len,bool eof,Buffer *list,
       return skip_len;
    if(skip_len>0 && eol<less)
       return skip_len;
+   if(end-less-1>=3 && less[1]=='!' && less[2]=='-' && less[3]=='-')
+   {
+      // found comment
+      if(end-less-4<3)
+	 return less-buf;
+      const char *scan=less+4;
+      for(;;)
+      {
+	 const char *eoc=find_char(scan,end-scan,'>');
+	 if(!eoc)
+	 {
+	    if(eof)  // unterminated comment.
+	       return len;
+	    return less-buf;
+	 }
+	 if(eoc>=less+4+2 && eoc[-1]=='-' && eoc[-2]=='-')
+	    return eoc+1-buf;
+	 scan=eoc+1;
+      }
+   }
    // FIXME: a > sign can be inside quoted value. (?)
    const char *more=find_char(less+1,end-less-1,'>');
    if(more==0)
@@ -296,7 +318,8 @@ static int parse_html(const char *buf,int len,bool eof,Buffer *list,
    if(tag_scan->tag==0)
       return tag_len;	// not interesting
 
-   bool hftp=(prefix && !xstrcmp(prefix->proto,"hftp"));
+   bool hftp=(prefix && (!xstrcmp(prefix->proto,"hftp")
+		      || !xstrcmp(prefix->proto,"ftp")));
 
    // ok, found the target.
 
@@ -561,7 +584,7 @@ parse_url_again:
 		    &day,month_name,&year,&hour,&minute,size_str);
       if(n==6)
       {
-	 debug("apache listing matched");
+	 debug(_("apache listing matched"));
 	 goto got_info;
       }
 
@@ -571,24 +594,24 @@ parse_url_again:
       n=sscanf(str,"%30s %2d-%3s-%d",size_str,&day,month_name,&year);
       if(n==4 && (size_str[0]=='-' || is_ascii_digit(size_str[0])))
       {
-	 debug("unusual apache listing matched");
+	 debug(_("unusual apache listing matched"));
 	 goto got_info;
       }
 
       char size_unit[7];
-      long size;
+      long long size;
       char week_day[4];
       int second;
       // Netscape-Proxy 2.53
-      if(9==sscanf(str,"%ld %6s %3s %3s %d %2d:%2d:%2d %4d",&size,size_unit,
+      if(9==sscanf(str,"%lld %6s %3s %3s %d %2d:%2d:%2d %4d",&size,size_unit,
 	       week_day,month_name,&day,&hour,&minute,&second,&year))
       {
 	 if(!strcasecmp(size_unit,"bytes")
 	 || !strcasecmp(size_unit,"byte"))
-	    sprintf(size_str,"%ld",size);
+	    sprintf(size_str,"%lld",size);
 	 else
-	    sprintf(size_str,"%ld%s",size,size_unit);
-	 debug("Netscape-Proxy 2.53 listing matched");
+	    sprintf(size_str,"%lld%s",size,size_unit);
+	 debug(_("Netscape-Proxy 2.53 listing matched"));
 	 goto got_info;
       }
       n=sscanf(str,"%3s %3s %d %2d:%2d:%2d %4d %s",
@@ -598,7 +621,7 @@ parse_url_again:
 	 strcpy(size_str,"-");
 	 if(!is_directory)
 	    is_sym_link=true;
-	 debug("Netscape-Proxy 2.53 listing matched (dir/symlink)");
+	 debug(_("Netscape-Proxy 2.53 listing matched (dir/symlink)"));
 	 goto got_info;
       }
       if(n==8) // maybe squid's EPLF listing.
@@ -606,7 +629,7 @@ parse_url_again:
 	 // skip rest of line, because there may be href to link target.
 	 skip_len=eol-buf+eol_len;
 	 // no symlinks here.
-	 debug("squid EPLF listing matched");
+	 debug(_("squid EPLF listing matched"));
 	 goto got_info;
       }
 
@@ -628,12 +651,12 @@ parse_url_again:
 	    strcpy(size_str,"-");
 	 }
 	 month--;
-	 debug("Mini-Proxy web server listing matched");
+	 debug(_("Mini-Proxy web server listing matched"));
 	 goto got_info;
       }
 
       // Apache Unix-like listing (from apache proxy):
-      //   Perms Nlnk user group size Mon DD (YYYY or hh:mm)
+      //   Perms Nlnk user [group] size Mon DD (YYYY or hh:mm)
       int perms_code;
       int n_links;
       char user[32];
@@ -641,13 +664,19 @@ parse_url_again:
       char year_or_time[6];
       int consumed;
 
-      n=sscanf(buf,"%10s %d %31s %31s %ld %3s %2d %5s%n",perms,&n_links,
+      n=sscanf(buf,"%10s %d %31s %31s %lld %3s %2d %5s%n",perms,&n_links,
 	    user,group,&size,month_name,&day,year_or_time,&consumed);
-      if(n==8 && -1!=(perms_code=parse_perms(perms+1))
+      if(n==4) // bsd-like listing without group?
+      {
+	 group[0]=0;
+	 n=sscanf(buf,"%10s %d %31s %lld %3s %2d %5s%n",perms,&n_links,
+	       user,&size,month_name,&day,year_or_time,&consumed);
+      }
+      if(n>=7 && -1!=(perms_code=parse_perms(perms+1))
       && -1!=(month=parse_month(month_name))
       && -1!=parse_year_or_time(year_or_time,&year,&hour,&minute))
       {
-	 sprintf(size_str,"%ld",size);
+	 sprintf(size_str,"%lld",size);
 	 if(perms[0]=='d')
 	    is_directory=true;
 	 else if(perms[0]=='l')
@@ -662,7 +691,7 @@ parse_url_again:
 	 }
 	 info_string=buf;
 	 info_string_len=consumed;
-	 debug("apache ftp over http proxy listing matched");
+	 debug(_("apache ftp over http proxy listing matched"));
 	 goto got_info;
       }
       perms[0]=0;
@@ -835,7 +864,7 @@ int HttpDirList::Do()
       {
 	 session->Open(curr,mode);
 	 session->UseCache(use_cache);
-	 ubuf=new FileInputBuffer(session);
+	 ubuf=new IOBufferFileAccess(session);
 	 if(LsCache::IsEnabled())
 	    ubuf->Save(LsCache::SizeLimit());
       }
@@ -929,8 +958,8 @@ const char *HttpDirList::Status()
    static char s[256];
    if(ubuf && !ubuf->Eof() && session->IsOpen())
    {
-      sprintf(s,_("Getting file list (%ld) [%s]"),
-		     session->GetPos(),session->CurrentStatus());
+      sprintf(s,_("Getting file list (%lld) [%s]"),
+		     (long long)session->GetPos(),session->CurrentStatus());
       return s;
    }
    return "";
@@ -1021,8 +1050,8 @@ int HttpGlob::Do()
    if(!ubuf)
    {
       curr_dir=0;
-      if(dir_list)
-	 curr_dir=dir_list[dir_index];
+      if(dir_list && (*dir_list)[dir_index])
+	 curr_dir=(*dir_list)[dir_index]->name;
       else if(dir_index==0)
 	 curr_dir=dir;
       if(curr_dir==0) // all done
@@ -1045,7 +1074,7 @@ int HttpGlob::Do()
 	 session->Open(curr_dir,FA::LONG_LIST);
 	 session->UseCache(use_cache);
 	 session->RereadManual();
-	 ubuf=new FileInputBuffer(session);
+	 ubuf=new IOBufferFileAccess(session);
 	 if(LsCache::IsEnabled())
 	    ubuf->Save(LsCache::SizeLimit());
       }
@@ -1096,7 +1125,10 @@ int HttpGlob::Do()
 
    set.rewind();
    for(FileInfo *f=set.curr(); f; f=set.next())
-      add(dir_file(curr_dir,f->name));
+   {
+      f->SetName(dir_file(curr_dir,f->name));
+      add(f);
+   }
 
    dir_index++;
    m=MOVED;
@@ -1112,8 +1144,8 @@ const char *HttpGlob::Status()
    static char s[256];
    if(ubuf && !ubuf->Eof() && session->IsOpen())
    {
-      sprintf(s,_("Getting file list (%ld) [%s]"),
-		     session->GetPos(),session->CurrentStatus());
+      sprintf(s,_("Getting file list (%lld) [%s]"),
+		     (long long)session->GetPos(),session->CurrentStatus());
       return s;
    }
    return "";
@@ -1153,7 +1185,7 @@ int HttpListInfo::Do()
       {
 	 session->Open("",FA::LONG_LIST);
 	 session->UseCache(use_cache);
-	 ubuf=new FileInputBuffer(session);
+	 ubuf=new IOBufferFileAccess(session);
 	 if(LsCache::IsEnabled())
 	    ubuf->Save(LsCache::SizeLimit());
       }
@@ -1309,12 +1341,14 @@ const char *HttpListInfo::Status()
    static char s[256];
    if(ubuf && !ubuf->Eof() && session->IsOpen())
    {
-      sprintf(s,_("Getting directory contents (%ld)"),session->GetPos());
+      sprintf(s,_("Getting directory contents (%lld)"),
+		     (long long)session->GetPos());
       return s;
    }
    if(get_info)
    {
-      sprintf(s,_("Getting files information (%d%%)"),session->InfoArrayPercentDone());
+      sprintf(s,_("Getting files information (%d%%)"),
+		     session->InfoArrayPercentDone());
       return s;
    }
    return "";
