@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LocalAccess.cc,v 1.24.2.1 2000/06/08 10:13:31 lav Exp $ */
+/* $Id: LocalAccess.cc,v 1.30 2000/09/07 18:38:36 lav Exp $ */
 
 /* this is not very useful, just a proof of concept */
 /* this is used in mirror command to build local files list */
@@ -596,7 +596,7 @@ int LocalGlob::Do()
    char *oldcwd=xgetcwd();
    if(!oldcwd)
    {
-      SetError("cannot get current directory");
+      SetError(_("cannot get current directory"));
       return MOVED;
    }
    if(chdir(cwd)==-1)
@@ -614,14 +614,19 @@ int LocalGlob::Do()
    for(unsigned i=0; i<g.gl_pathc; i++)
    {
       struct stat st;
+      FileInfo info(g.gl_pathv[i]);
       if(stat(g.gl_pathv[i],&st)!=-1)
       {
 	 if(dirs_only && !S_ISDIR(st.st_mode))
 	    continue;
 	 if(files_only && !S_ISREG(st.st_mode))
 	    continue;
+	 if(S_ISDIR(st.st_mode))
+	    info.SetType(info.DIRECTORY);
+	 else if(S_ISREG(st.st_mode))
+	    info.SetType(info.NORMAL);
       }
-      add(g.gl_pathv[i]);
+      add(&info);
    }
    globfree(&g);
 
@@ -643,11 +648,12 @@ LocalDirList::LocalDirList(ArgV *a,const char *cwd)
    : DirList(a)
 {
    fg_data=0;
+   a->setarg(0,"ls");
    a->insarg(1,"-l");
    InputFilter *f=new InputFilter(a);
    f->SetCwd(cwd);
    Delete(buf);
-   buf=new FileInputBuffer(f);
+   buf=new IOBufferFDStream(f,IOBuffer::GET);
 }
 int LocalDirList::Do()
 {
@@ -675,7 +681,8 @@ LocalDirList::~LocalDirList()
       delete fg_data;
 }
 
-#ifdef MODULE
+#include "modconfig.h"
+#ifdef MODULE_PROTO_FILE
 CDECL void module_init()
 {
    LocalAccess::ClassInit();
