@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.47.2.1 2000/07/12 12:00:06 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.59 2000/09/29 07:51:21 lav Exp $ */
 
 #include <config.h>
 
@@ -37,6 +37,7 @@
 #include "plural.h"
 #include "getopt.h"
 #include "FindJob.h"
+#include "url.h"
 
 void  MirrorJob::PrintStatus(int v)
 {
@@ -116,8 +117,7 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
    case(WAITING_FOR_MKDIR_BEFORE_SUBMIRROR):
    case(REMOTE_REMOVE_OLD):
    case(REMOTE_CHMOD):
-      if(waiting && !waiting->Done())
-	 waiting->ShowRunStatus(s);
+      Job::ShowRunStatus(s);
       break;
 
    case(MAKE_REMOTE_DIR):
@@ -180,9 +180,10 @@ void  MirrorJob::HandleFile(int how)
 		     goto skip;
 		  }
 	       }
-	       waiting=new rmJob(Clone(),args);
-	       waiting->SetParentFg(this);
-	       waiting->cmdline=args->Combine();
+	       Job *j=new rmJob(Clone(),args);
+	       j->SetParentFg(this);
+	       j->cmdline=args->Combine();
+	       AddWaiting(j);
 	       mod_files++;
 	    }
 	    else
@@ -250,8 +251,7 @@ void  MirrorJob::HandleFile(int how)
 	 }
 
 	 FileCopyPeerFA *src_peer=
-	    new FileCopyPeerFA(session,file->name,FA::RETRIEVE);
-	 src_peer->DontReuseSession(); // mirror will need this session
+	    new FileCopyPeerFA(session->Clone(),file->name,FA::RETRIEVE);
 	 FileCopyPeer *dst_peer=
 	    FileCopyPeerFDStream::NewPut(local_name,cont_this);
 
@@ -264,10 +264,10 @@ void  MirrorJob::HandleFile(int how)
 	    cp->SetDate(file->date);
 	 if(file->defined&file->SIZE)
 	    cp->SetSize(file->size);
-	 waiting=cp;
-	 waiting->SetParentFg(this);
-	 waiting->cmdline=(char*)xmalloc(10+strlen(file->name));
-	 sprintf(waiting->cmdline,"\\get %s",file->name);
+	 AddWaiting(cp);
+	 cp->SetParentFg(this);
+	 cp->cmdline=(char*)xmalloc(10+strlen(file->name));
+	 sprintf(cp->cmdline,"\\get %s",file->name);
 	 break;
       }
       case(FileInfo::DIRECTORY):
@@ -289,9 +289,10 @@ void  MirrorJob::HandleFile(int how)
 		  args=new ArgV("mkdir");
 		  args->Append("--");
 		  args->Append(file->name);
-		  waiting=new mkdirJob(Clone(),args);
-		  waiting->SetParentFg(this);
-		  waiting->cmdline=args->Combine();
+		  Job *j=new mkdirJob(Clone(),args);
+		  j->SetParentFg(this);
+		  j->cmdline=args->Combine();
+		  AddWaiting(j);
 	       }
 	       else
 		  dir_made=true;
@@ -333,11 +334,11 @@ void  MirrorJob::HandleFile(int how)
 	 // launch sub-mirror
 	 MirrorJob *mj=new MirrorJob(Clone(),local_name,remote_name);
 	 mj->parent_mirror=this;
-	 waiting=mj;
-	 waiting->SetParentFg(this);
-	 waiting->cmdline=(char*)xmalloc(strlen("\\mirror")+1+
+	 AddWaiting(mj);
+	 mj->SetParentFg(this);
+	 mj->cmdline=(char*)xmalloc(strlen("\\mirror")+1+
 					 strlen(file->name)+1);
-	 sprintf(waiting->cmdline,"\\mirror %s",file->name);
+	 sprintf(mj->cmdline,"\\mirror %s",file->name);
 
 	 // inherit flags and other things
 	 mj->SetFlags(flags,1);
@@ -354,6 +355,8 @@ void  MirrorJob::HandleFile(int how)
 
 	 mj->verbose_report=verbose_report;
 	 mj->newer_than=newer_than;
+	 mj->parallel=parallel;
+	 mj->remove_source_files=remove_source_files;
 
 	 if(verbose_report>=3)
 	    Report(_("Mirroring directory `%s'"),
@@ -416,7 +419,8 @@ void  MirrorJob::HandleFile(int how)
    return;
 
 skip:
-   to_transfer->next();
+   if(how==1)  // NOTE: check invocation places before changing this
+      to_transfer->next();
    return;
 }
 
@@ -447,6 +451,7 @@ int   MirrorJob::Do()
    int	 res;
    int	 m=STALL;
    FileInfo *fi;
+   Job	 *j;
 
    switch(state)
    {
@@ -459,7 +464,7 @@ int   MirrorJob::Do()
 	 local_session=FileAccess::New("file");
 	 if(!local_session)
 	 {
-	    eprintf("mirror: cannot create `file:' access object, installation error?\n");
+	    eprintf(_("mirror: cannot create `file:' access object, installation error?\n"));
 	    state=DONE;
 	    return MOVED;
 	 }
@@ -493,6 +498,7 @@ int   MirrorJob::Do()
 	 return MOVED;
       }
       session->Chdir(remote_dir);
+      redirections=0;
       Roll(session);
       state=CHANGING_REMOTE_DIR;
       return MOVED;
@@ -511,7 +517,38 @@ int   MirrorJob::Do()
 	 return STALL;
       if(res<0)
       {
+	 if(res==FA::FILE_MOVED)
+	 {
+	    // cd to another url.
+	    const char *loc_c=session->GetNewLocation();
+	    int max_redirections=ResMgr::Query("xfer:max-redirections",0);
+	    if(loc_c && loc_c[0] && max_redirections>0
+	    && loc_c[strlen(loc_c)-1]=='/')
+	    {
+	       if(++redirections>max_redirections)
+		  goto cd_err_normal;
+	       eprintf(_("%s: received redirection to `%s'\n"),"mirror",loc_c);
+
+	       char *loc=alloca_strdup(loc_c);
+	       session->Close(); // loc_c is no longer valid.
+
+	       ParsedURL u(loc,true);
+
+	       if(!u.proto)
+	       {
+		  url::decode_string(loc);
+		  session->Chdir(loc);
+		  return MOVED;
+	       }
+	       SessionPool::Reuse(session);
+	       session=FA::New(&u);
+	       session->Chdir(u.path);
+	       return MOVED;
+	    }
+	 }
+      cd_err_normal:
 	 eprintf("mirror: %s\n",session->StrError(res));
+	 error_count++;
 	 state=DONE;
 	 session->Close();
 	 return MOVED;
@@ -548,6 +585,7 @@ int   MirrorJob::Do()
       {
       list_info_error:
 	 eprintf("mirror: %s\n",list_info->ErrorText());
+	 error_count++;
 	 state=DONE;
       	 Delete(list_info);
 	 list_info=0;
@@ -568,19 +606,23 @@ int   MirrorJob::Do()
 	 InitSets(remote_set,local_set);
 
       to_transfer->rewind();
-      waiting=0;
+//       waiting=0; ???
       state=WAITING_FOR_SUBGET;
       return MOVED;
    }
 
    case(WAITING_FOR_RM_BEFORE_PUT):
    {
-      if(waiting)
+      if(waiting_num>0)
       {
-	 if(!waiting->Done())
-	    return STALL;
-	 Delete(waiting);
-	 waiting=0;
+	 while((j=FindDoneAwaitedJob())!=0)
+	 {
+	    m=MOVED;
+	    RemoveWaiting(j);
+	    Delete(j);
+	 }
+	 if(waiting_num>0)
+	    return m;
       }
       Report(_("Sending local file `%s'"),
 	       dir_file(local_relative_dir,file->name));
@@ -603,8 +645,7 @@ int   MirrorJob::Do()
 #endif
 
       FileCopyPeerFA *dst_peer=
-	 new FileCopyPeerFA(session,file->name,FA::STORE);
-      dst_peer->DontReuseSession(); // mirror won't need session
+	 new FileCopyPeerFA(session->Clone(),file->name,FA::STORE);
       FileCopyPeer *src_peer=
 	 FileCopyPeerFDStream::NewGet(local_name);
 
@@ -613,44 +654,50 @@ int   MirrorJob::Do()
 	 c->RemoveSourceLater();
       CopyJob *cp=
 	 new CopyJob(c,file->name,"mirror");
-      waiting=cp;
-      waiting->SetParentFg(this);
-      waiting->cmdline=(char*)xmalloc(10+strlen(file->name));
-      sprintf(waiting->cmdline,"\\put %s",file->name);
+      AddWaiting(cp);
+      cp->SetParentFg(this);
+      cp->cmdline=(char*)xmalloc(10+strlen(file->name));
+      sprintf(cp->cmdline,"\\put %s",file->name);
       state=WAITING_FOR_SUBGET;
       return MOVED;
    }
 
    case(WAITING_FOR_SUBGET):
-      if(waiting && waiting->Done())
+      j=FindDoneAwaitedJob();
+      if(j)
       {
-	 to_transfer->next();
-	 Delete(waiting);
-	 waiting=0;
+	 if(j->ExitCode()!=0)
+	    error_count++;
+	 RemoveWaiting(j);
+	 Delete(j);
       }
-      while(!waiting && state==WAITING_FOR_SUBGET)
+      while(waiting_num<parallel && state==WAITING_FOR_SUBGET)
       {
 	 file=to_transfer->curr();
       	 if(!file)
 	 {
+	    if(waiting_num>0)
+	       return m;
 	    to_transfer->rewind();
 	    state=WAITING_FOR_SUBMIRROR;
 	    return MOVED;
 	 }
 	 HandleFile(0);
+	 to_transfer->next();
 	 m=MOVED;
       }
       return m;
 
    case(WAITING_FOR_MKDIR_BEFORE_SUBMIRROR):
-      if(waiting)
+      j=FindDoneAwaitedJob();
+      if(j==0 && waiting_num>0)
+	 return STALL;
+      if(j)
       {
-	 if(!waiting->Done())
-	    return STALL;
-	 if(waiting->ExitCode()==0)
+	 if(j->ExitCode()==0)
 	    dir_made=true;
-	 Delete(waiting);
-	 waiting=0;
+	 RemoveWaiting(j);
+	 Delete(j);
 	 if(!dir_made)
 	    to_transfer->next();
       }
@@ -658,9 +705,12 @@ int   MirrorJob::Do()
       return MOVED;
 
    case(WAITING_FOR_SUBMIRROR):
-      if(waiting && waiting->Done())
+      j=FindDoneAwaitedJob();
+      if(j==0 && waiting_num>0)
+	 return STALL;
+      if(j)
       {
-	 MirrorJob &mj=*(MirrorJob*)waiting; // we are sure it is a MirrorJob
+	 MirrorJob &mj=*(MirrorJob*)j; // we are sure it is a MirrorJob
 	 tot_files+=mj.tot_files;
 	 new_files+=mj.new_files;
 	 mod_files+=mj.mod_files;
@@ -671,12 +721,13 @@ int   MirrorJob::Do()
 	 del_symlinks+=mj.del_symlinks;
 	 dirs+=mj.dirs;
 	 del_dirs+=mj.del_dirs;
+	 error_count+=mj.error_count;
 
 	 to_transfer->next();
-	 Delete(waiting);
-	 waiting=0;
+	 RemoveWaiting(j);
+	 Delete(j);
       }
-      while(!waiting && state==WAITING_FOR_SUBMIRROR)
+      while(waiting_num<1 && state==WAITING_FOR_SUBMIRROR)
       {
 	 file=to_transfer->curr();
       	 if(!file)
@@ -734,7 +785,7 @@ int   MirrorJob::Do()
       return m;
 
    case(REMOTE_REMOVE_OLD):
-      if(!waiting)
+      if(waiting_num==0)
       {
 	 if(flags&DELETE)
 	 {
@@ -749,16 +800,17 @@ int   MirrorJob::Do()
 		  Report(_("Removing old remote directory `%s'"),
 			   dir_file(remote_relative_dir,file->name));
 		  args->getnext(); // prepare args position.
-	       	  waiting=new FinderJob_Cmd(Clone(),args,FinderJob_Cmd::RM);
+	       	  j=new FinderJob_Cmd(Clone(),args,FinderJob_Cmd::RM);
 	       }
 	       else
 	       {
 		  Report(_("Removing old remote file `%s'"),
 			   dir_file(remote_relative_dir,file->name));
-		  waiting=new rmJob(Clone(),args);
+		  j=new rmJob(Clone(),args);
 	       }
-	       waiting->SetParentFg(this);
-	       waiting->cmdline=args->Combine();
+	       j->SetParentFg(this);
+	       j->cmdline=args->Combine();
+	       AddWaiting(j);
 	    }
 	 }
 	 else if(flags&REPORT_NOT_DELETED)
@@ -769,13 +821,14 @@ int   MirrorJob::Do()
 			dir_file(remote_relative_dir,file->name));
 	    }
 	 }
-	 if(!waiting)
+	 if(waiting_num==0)
 	    goto pre_REMOTE_CHMOD;
       }
-      if(waiting->Done())
+      j=FindDoneAwaitedJob();
+      if(j)
       {
-	 Delete(waiting);
-	 waiting=0;
+	 RemoveWaiting(j);
+	 Delete(j);
 	 return MOVED;
       }
       return m;
@@ -791,10 +844,11 @@ int   MirrorJob::Do()
       m=MOVED;
       goto remote_chmod_next;
    case(REMOTE_CHMOD):
-      if(waiting->Done())
+      j=FindDoneAwaitedJob();
+      if(j)
       {
-	 Delete(waiting);
-	 waiting=0;
+	 RemoveWaiting(j);
+	 Delete(j);
 
       remote_chmod_next:
 	 fi=to_transfer->curr();
@@ -809,9 +863,9 @@ int   MirrorJob::Do()
 	 ArgV *a=new ArgV("chmod");
 	 a->Append(fi->name);
 	 ChmodJob *cj=new ChmodJob(Clone(),fi->mode&~get_mode_mask(),a);
-	 waiting=cj;
-	 waiting->SetParentFg(this);
-	 waiting->cmdline=a->Combine();
+	 AddWaiting(cj);
+	 cj->SetParentFg(this);
+	 cj->cmdline=a->Combine();
 	 cj->BeQuiet();   // chmod is not supported on all servers; be quiet.
 
 	 m=MOVED;
@@ -843,6 +897,7 @@ MirrorJob::MirrorJob(FileAccess *f,const char *new_local_dir,const char *new_rem
    tot_files=new_files=mod_files=del_files=
    tot_symlinks=new_symlinks=mod_symlinks=del_symlinks=0;
    dirs=0; del_dirs=0;
+   error_count=0;
 
    flags=0;
 
@@ -862,6 +917,10 @@ MirrorJob::MirrorJob(FileAccess *f,const char *new_local_dir,const char *new_rem
 
    use_cache=false;
    remove_source_files=false;
+
+   parallel=1;
+
+   redirections=0;
 }
 
 MirrorJob::~MirrorJob()
@@ -993,6 +1052,7 @@ CMD(mirror)
       {"dereference",no_argument,0,'L'},
       {"use-cache",no_argument,0,256+'C'},
       {"Remove-source-files",no_argument,0,256+'R'},
+      {"parallel",optional_argument,0,'P'},
       {0}
    };
 
@@ -1035,9 +1095,10 @@ CMD(mirror)
    int	 verbose=0;
    const char *newer_than=0;
    bool  remove_source_files=false;
+   int	 parallel=0;
 
    args->rewind();
-   while((opt=args->getopt_long("esi:x:t:T:nrpcRvN:L",mirror_opts,0))!=EOF)
+   while((opt=args->getopt_long("esi:x:t:T:nrpcRvN:LP",mirror_opts,0))!=EOF)
    {
       switch(opt)
       {
@@ -1107,6 +1168,12 @@ CMD(mirror)
       case(256+'R'):
 	 remove_source_files=true;
 	 break;
+      case('P'):
+	 if(optarg)
+	    parallel=atoi(optarg);
+	 else
+	    parallel=3;
+	 break;
       case('?'):
 	 parent->eprintf(_("Try `help %s' for more information.\n"),args->a0());
 	 return 0;
@@ -1200,6 +1267,12 @@ CMD(mirror)
    j->UseCache(use_cache);
    if(remove_source_files)
       j->RemoveSourceFiles();
+   if(parallel<0)
+      parallel=0;
+   if(parallel>16)
+      parallel=16;   // a sane limit.
+   if(parallel)
+      j->SetParallel(parallel);
    return j;
 
 err_out:
@@ -1223,7 +1296,8 @@ mode_t MirrorJob::get_mode_mask()
    return mode_mask;
 }
 
-#ifdef MODULE
+#include "modconfig.h"
+#ifdef MODULE_CMD_MIRROR
 CDECL void module_init()
 {
    CmdExec::RegisterCommand("mirror",cmd_mirror);
