@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.28 2000/03/15 16:44:22 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.35 2000/07/25 17:27:19 lav Exp $ */
 
 #include <config.h>
 
@@ -37,7 +37,8 @@ void NetAccess::Init()
    idle=0;
    idle_start=now;
    max_retries=0;
-   retries=0;
+   max_persist_retries=0;
+   persist_retries=0;
    socket_buffer=0;
    socket_maxseg=0;
 
@@ -100,6 +101,7 @@ void NetAccess::Reconfig(const char *name)
       reconnect_interval_max=reconnect_interval;
    idle = ResMgr::Query("net:idle",c);
    max_retries = ResMgr::Query("net:max-retries",c);
+   max_persist_retries = ResMgr::Query("net:persist-retries",c);
    socket_buffer = ResMgr::Query("net:socket-buffer",c);
    socket_maxseg = ResMgr::Query("net:socket-maxseg",c);
    connection_limit = ResMgr::Query("net:connection-limit",c);
@@ -154,7 +156,7 @@ const char *NetAccess::SocketNumericAddress(const sockaddr_u *u)
 {
 #ifdef HAVE_GETNAMEINFO
    static char buf[NI_MAXHOST];
-   if(getnameinfo(&u->sa,sizeof(*u),buf,sizeof(buf),0,0,NI_NUMERICHOST)<0)
+   if(getnameinfo(&u->sa,SocketAddrLen(u),buf,sizeof(buf),0,0,NI_NUMERICHOST)<0)
       return "????";
    return buf;
 #else
@@ -304,7 +306,9 @@ int NetAccess::Resolve(const char *defp,const char *ser,const char *pr)
 
    if(!resolver)
    {
-      ClearPeer();
+      xfree(peer);
+      peer=0;
+      peer_num=0;
       if(proxy)
 	 resolver=new Resolver(proxy,proxy_port,defp);
       else
@@ -334,7 +338,8 @@ int NetAccess::Resolve(const char *defp,const char *ser,const char *pr)
    peer=(sockaddr_u*)xmalloc(resolver->GetResultSize());
    peer_num=resolver->GetResultNum();
    resolver->GetResult(peer);
-   peer_curr=0;
+   if(peer_curr>=peer_num)
+      peer_curr=0;
 
    Delete(resolver);
    resolver=0;
@@ -360,6 +365,7 @@ RateLimit::~RateLimit()
 }
 
 #define LARGE 0x10000000
+#define DEFAULT_MAX_COEFF 2
 void RateLimit::BytesPool::AdjustTime()
 {
    if(SMTask::now>t)
@@ -368,12 +374,14 @@ void RateLimit::BytesPool::AdjustTime()
 
       // prevent overflow
       if((LARGE-pool)/dif < rate)
-	 pool = pool_max>0 ? pool_max : LARGE;
+	 pool = pool_max>0 ? pool_max : rate*DEFAULT_MAX_COEFF;
       else
 	 pool += dif*rate;
 
       if(pool>pool_max && pool_max>0)
 	 pool=pool_max;
+      if(pool_max==0 && pool>rate*DEFAULT_MAX_COEFF)
+	 pool=rate*DEFAULT_MAX_COEFF;
 
       t=SMTask::now;
    }
@@ -494,8 +502,8 @@ bool NetAccess::NextTry()
    }
    retries++;
 
-   assert(peer!=0);
-   assert(peer_curr<peer_num);
+//    assert(peer!=0);
+//    assert(peer_curr<peer_num);
 
    return true;
 }
@@ -503,6 +511,7 @@ bool NetAccess::NextTry()
 void NetAccess::Close()
 {
    retries=0;
+   persist_retries=0;
    takeover_time=NO_DATE;
 
    Delete(resolver);
