@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.h,v 1.15 2000/03/15 16:44:22 lav Exp $ */
+/* $Id: NetAccess.h,v 1.17 2000/06/22 23:49:33 lav Exp $ */
 
 #ifndef NETACCESS_H
 #define NETACCESS_H
@@ -38,7 +38,8 @@ protected:
    void	 NextPeer();
 
    int	 max_retries;
-   int	 retries;
+   int	 max_persist_retries;
+   int	 persist_retries;
 
    int	 idle;
    time_t idle_start;
@@ -101,10 +102,6 @@ public:
    void Connect(const char *,const char *);
    void ConnectVerify();
 
-   int GetRetries() { return retries; }
-   void SetRetries(int r) { retries=r; }
-   time_t GetTryTime() { return try_time; }
-
    void Close();
 
    int CountConnections();
