@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.21 2000/04/03 18:29:38 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.26 2000/08/26 19:11:24 lav Exp $ */
 
 #include <config.h>
 
@@ -26,6 +26,7 @@
 #include <ctype.h>
 #include <unistd.h>
 #include <math.h>
+#include <regex.h>
 #include "ResMgr.h"
 #include "SMTask.h"
 #include "xmalloc.h"
@@ -197,7 +198,7 @@ char *ResMgr::Format(bool with_defaults,bool only_defaults)
       {
 	 size+=4+strlen(scan->type->name);
 	 if(scan->closure)
-	    size+=1+2*strlen(scan->closure);
+	    size+=1+1+2*strlen(scan->closure)+1;
 	 size+=1+1+2*strlen(scan->value)+1+1;
 	 n++;
       }
@@ -255,18 +256,25 @@ char *ResMgr::Format(bool with_defaults,bool only_defaults)
       if(s)
       {
 	 *store++='/';
+	 bool par=false;
+	 if(strcspn(s," \t>|;&")!=strlen(s))
+	    par=true;
+	 if(par)
+	    *store++='"';
 	 while(*s)
 	 {
-	    if(strchr("\" \t\\>|",*s))
+	    if(strchr("\"\\",*s))
 	       *store++='\\';
 	    *store++=*s++;
 	 }
+	 if(par)
+	    *store++='"';
       }
       *store++=' ';
       s=arr[i]->value;
 
       bool par=false;
-      if(*s==0 || strcspn(s," \t>|")!=strlen(s))
+      if(*s==0 || strcspn(s," \t>|;&")!=strlen(s))
 	 par=true;
       if(par)
 	 *store++='"';
@@ -378,6 +386,31 @@ bool ResMgr::Resource::ClosureMatch(const char *cl_data)
    return 0==fnmatch(closure,cl_data,FNM_PATHNAME);
 }
 
+const char *ResMgr::QueryNext(const char *name,const char **closure,Resource **ptr)
+{
+   ResDecl *type;
+   // find type of given variable
+   const char *msg=FindVar(name,&type);
+   if(msg)
+      return 0;
+
+   if(*ptr==0)
+      *ptr=chain;
+   else
+      *ptr=(*ptr)->next;
+
+   while(*ptr)
+   {
+      if((*ptr)->type==type)
+      {
+	 *closure=(*ptr)->closure;
+	 return (*ptr)->value;
+      }
+      *ptr=(*ptr)->next;
+   }
+   return 0;
+}
+
 const char *ResMgr::SimpleQuery(const char *name,const char *closure)
 {
    const char *msg;
@@ -388,15 +421,15 @@ const char *ResMgr::SimpleQuery(const char *name,const char *closure)
    if(msg)
       return 0;
 
-   Resource **scan;
+   Resource *scan;
    // find the value
-   for(scan=&chain; *scan; scan=&(*scan)->next)
-      if((*scan)->type==type && (*scan)->ClosureMatch(closure))
+   for(scan=chain; scan; scan=scan->next)
+      if(scan->type==type && scan->ClosureMatch(closure))
 	 break;
 
    // if found
-   if(*scan)
-      return (*scan)->value;
+   if(scan)
+      return scan->value;
 
    return 0;
 }
@@ -411,7 +444,7 @@ ResValue ResMgr::Query(const char *name,const char *closure)
    if(msg)
    {
       // debug only
-      fprintf(stderr,"Query of variable `%s' failed: %s\n",name,msg);
+      fprintf(stderr,_("Query of variable `%s' failed: %s\n"),name,msg);
       return 0;
    }
 
@@ -584,3 +617,24 @@ const char *ResMgr::RangeValidate(char **s)
       return r.ErrorText();
    return 0;
 }
+
+const char *ResMgr::ERegExpValidate(char **s)
+{
+   if(**s==0)
+      return 0;
+   regex_t re;
+   int err=regcomp(&re,*s,REG_EXTENDED|REG_NOSUB);
+   if(err)
+   {
+      static char err_msg[128];
+      regerror(err,0,err_msg,sizeof(err_msg));
+      return err_msg;
+   }
+   regfree(&re);
+   return 0;
+}
+
+const char *ResMgr::NoClosure(char **)
+{
+   return _("no closure defined for this setting");
+}
