@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.h,v 1.15 2000/04/15 18:57:30 lav Exp $ */
+/* $Id: ResMgr.h,v 1.19 2000/08/23 23:17:46 lav Exp $ */
 
 #ifndef RESMGR_H
 #define RESMGR_H
@@ -32,10 +32,12 @@ typedef const char *ResValValid(char **value);
 typedef const char *ResClValid(char **closure);
 
 class ResValue;
+class ResDecl;
 
 class ResMgr
 {
    friend class ResDecl;
+public:
    class Resource
    {
       friend class ResMgr;
@@ -63,10 +65,12 @@ class ResMgr
       }
    };
 
+private:
    static Resource *chain;
    static ResDecl *type_chain;
 
 public:
+   static const char *QueryNext(const char *name,const char **closure,Resource **ptr);
    static const char *SimpleQuery(const char *name,const char *closure);
    static ResValue Query(const char *name,const char *closure);
 
@@ -85,6 +89,8 @@ public:
    static const char *FloatValidate(char **value);
    static const char *TimeIntervalValidate(char **value);
    static const char *RangeValidate(char **value);
+   static const char *ERegExpValidate(char **value);
+   static const char *NoClosure(char **);
    static bool str2bool(const char *value);
 
    static void ClassInit();
