@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Resolver.cc,v 1.30.2.1 2000/09/07 15:20:59 lav Exp $ */
+/* $Id: Resolver.cc,v 1.38 2000/09/05 12:46:48 lav Exp $ */
 
 #include <config.h>
 
@@ -75,11 +75,11 @@ CDECL int res_search(const char*,int,int,unsigned char*,int);
 static ResDecl
    res_cache_enable("dns:cache-enable", "yes", ResMgr::BoolValidate,0),
    res_cache_expire("dns:cache-expire", "24h", ResMgr::TimeIntervalValidate,0),
-   res_cache_size  ("dns:cache-size",   "256", ResMgr::UNumberValidate,0),
+   res_cache_size  ("dns:cache-size",   "256", ResMgr::UNumberValidate,ResMgr::NoClosure),
    res_timeout	   ("dns:fatal-timeout","0",   ResMgr::UNumberValidate,0),
    res_order	   ("dns:order",	DEFAULT_ORDER, Resolver::OrderValidate,0),
    res_query_srv   ("dns:SRV-query",    "no",  ResMgr::BoolValidate,0),
-   res_use_fork	   ("dns:use-fork",     "yes", ResMgr::BoolValidate,0);
+   res_use_fork	   ("dns:use-fork",     "yes", ResMgr::BoolValidate,ResMgr::NoClosure);
 
 
 struct address_family
@@ -163,7 +163,7 @@ int   Resolver::Do()
       cache->Find(hostname,portname,defport,service,proto,&a,&n);
       if(a && n>0)
       {
-	 Log::global->Write(10,"dns cache hit\n");
+	 Log::global->Write(10,_("dns cache hit\n"));
 	 addr_num=n;
 	 addr=(sockaddr_u*)xmalloc(n*sizeof(*addr));
 	 memcpy(addr,a,n*sizeof(*addr));
@@ -224,7 +224,7 @@ int   Resolver::Do()
       }
       if(!buf)
       {
-	 buf=new FileInputBuffer(new FDStream(pipe_to_child[0],"<pipe-in>"));
+	 buf=new IOBufferFDStream(new FDStream(pipe_to_child[0],"<pipe-in>"),IOBuffer::GET);
 // 	 Roll(buf);
 	 m=MOVED;
       }
@@ -488,7 +488,7 @@ void Resolver::LookupSRV_RR()
 	    return;
       }
       time(&try_time);
-      len=res_search(srv_name, C_IN, T_SRV, (u_char*)answer, sizeof(answer));
+      len=res_search(srv_name, C_IN, T_SRV, answer, sizeof(answer));
       if(len>=0)
 	 break;
 #ifdef HAVE_H_ERRNO
@@ -663,14 +663,87 @@ void Resolver::LookupOne(const char *name)
 
       time(&try_time);
 
+#if defined(HAVE_GETADDRINFO) && INET6 \
+   && !defined(HAVE_GETHOSTBYNAME2) \
+   && !defined(HAVE_GETIPNODEBYNAME)
+
+      // getaddrinfo support by Brandon Hume
+      struct addrinfo	    *ainfo=0,
+			    *a_res,
+			    a_hint;
+      int		    ainfo_res;
+      struct sockaddr	    *sockname;
+      struct sockaddr_in    *inet_addr;
+      struct sockaddr_in6   *inet6_addr;
+      const char	    *addr_data;
+
+      a_hint.ai_flags	    = AI_PASSIVE;
+      a_hint.ai_family	    = PF_UNSPEC;
+      a_hint.ai_socktype    = 0;
+      a_hint.ai_protocol    = 0;
+      a_hint.ai_addrlen	    = 0;
+      a_hint.ai_canonname   = NULL;
+      a_hint.ai_addr	    = NULL;
+      a_hint.ai_next	    = NULL;
+
+      ainfo_res	= getaddrinfo(name, NULL, &a_hint, &ainfo);
+
+      if(ainfo_res == 0)
+      {
+        // by lav: add addresses in specified order.
+	for(int af=af_order[af_index]; af!=-1; af=af_order[++af_index])
+	{
+	    for(a_res = ainfo; a_res != NULL; a_res = a_res->ai_next)
+	    {
+	       if(a_res->ai_family!=af)
+		  continue;
+
+	       sockname	= a_res->ai_addr;
+
+	       switch(a_res->ai_family)
+	       {
+	       case AF_INET:
+		  inet_addr	= (sockaddr_in *)sockname;
+		  addr_data	= (const char *)&inet_addr->sin_addr.s_addr;
+		  break;
+	       case AF_INET6:
+		  inet6_addr	= (sockaddr_in6 *)sockname;
+		  addr_data	= (const char *)inet6_addr->sin6_addr.s6_addr;
+		  break;
+	       default:
+		  continue;
+	       }
+
+	       AddAddress(a_res->ai_family, addr_data, a_res->ai_addrlen);
+	    }
+	 }
+
+	 freeaddrinfo(ainfo);
+	 break;
+      }
+
+      if(ainfo_res != EAI_AGAIN)
+      {
+	 error = gai_strerror(ainfo_res);
+	 break;
+      }
+
+#else // !HAVE_GETADDRINFO
+
       int af=af_order[af_index];
       if(af==-1)
 	 break;
 
       struct hostent *ha;
-#ifdef HAVE_GETHOSTBYNAME2
+# if defined(HAVE_GETIPNODEBYNAME)
+#  ifndef HAVE_H_ERRNO
+#   define HAVE_H_ERRNO 1
+#  endif
+      int h_errno=0;
+      ha=getipnodebyname(name,af,0,&h_errno);
+# elif defined(HAVE_GETHOSTBYNAME2)
       ha=gethostbyname2(name,af);
-#else
+# else
       if(af==AF_INET)
 	 ha=gethostbyname(name);
       else
@@ -678,7 +751,7 @@ void Resolver::LookupOne(const char *name)
 	 af_index++;
 	 continue;
       }
-#endif
+# endif
 
       if(ha)
       {
@@ -688,21 +761,24 @@ void Resolver::LookupOne(const char *name)
 	 af_index++;
 	 continue;
       }
-#ifdef HAVE_H_ERRNO
+
+# ifdef HAVE_H_ERRNO
       if(h_errno!=TRY_AGAIN)
-#endif
+# endif
       {
 	 if(error==0)
 	 {
-#ifdef HAVE_H_ERRNO
+# ifdef HAVE_H_ERRNO
 	    error=hstrerror(h_errno);
-#else
-	    error="Host name lookup failure";
-#endif
+# else
+	    error=_("Host name lookup failure");
+# endif
 	 }
 	 af_index++;
 	 continue; // try other address families
       }
+#endif /* HAVE_GETADDRINFO */
+
       time_t t;
       if((t=time(0))-try_time<5)
 	 sleep(5-(t-try_time));
@@ -727,7 +803,7 @@ void Resolver::DoGethostbyname()
 	 {
 	    buf->Put("P");
 	    char *msg=string_alloca(64+strlen(tproto));
-	    sprintf(msg,"no such %s service",tproto);
+	    sprintf(msg,_("no such %s service"),tproto);
 	    buf->Put(msg);
 	    goto flush;
 	 }
@@ -746,13 +822,13 @@ void Resolver::DoGethostbyname()
       return;
 
    if(!buf)
-      buf=new FileOutputBuffer(new FDStream(pipe_to_child[1],"<pipe-out>"));
+      buf=new IOBufferFDStream(new FDStream(pipe_to_child[1],"<pipe-out>"),IOBuffer::PUT);
 
    if(addr_num==0)
    {
       buf->Put("E");
       if(error==0)
-	 error="No address found";
+	 error=_("No address found");
       buf->Put(error);
       goto flush;
    }
@@ -771,7 +847,7 @@ flush:
 
 void Resolver::Reconfig(const char *name)
 {
-   timeout = res_timeout.Query(0);
+   timeout = res_timeout.Query(hostname);
    if(!name || strncmp(name,"dns:",4))
       return;
    cache->Clear();
