@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: StatusLine.cc,v 1.5 1999/08/04 17:01:18 lav Exp $ */
+/* $Id: StatusLine.cc,v 1.7 2000/02/08 11:55:49 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -98,7 +98,7 @@ void StatusLine::Show(const char *f,...)
    {
       strcpy(to_be_shown,newstr);
       update_delayed=true;
-      SMTask::block+=TimeOut(1000);
+      TimeoutS(1);
    }
 }
 
@@ -197,6 +197,6 @@ int StatusLine::Do()
       update_time=now;
       return STALL;
    }
-   Timeout(1000);
+   TimeoutS(1);
    return STALL;
 }
