@@ -18,22 +18,31 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: fg.cc,v 1.2 1998/08/17 14:07:48 lav Exp $ */
+
 #include <config.h>
 #include <signal.h>
 #include <unistd.h>
 #include "fg.h"
 
+void FgData::cont()
+{
+   if(!pg)
+      return;
+   if(kill(-pg,SIGCONT)==-1)
+      kill(pg,SIGCONT);
+}
+
 FgData::FgData(pid_t npg,bool fg)
 {
    old_pgrp=0;
    pg=npg;
+   if(!pg)
+      return;
    if(fg)
       Fg();
    else
-   {
-      if(pg)
-	 kill(-pg,SIGCONT);
-   }
+      cont();
 }
 
 FgData::~FgData()
@@ -50,16 +59,21 @@ void FgData::Fg()
    if(tc_grp==-1 || tc_grp==getpgrp())
    {
       old_pgrp=getpgrp();
+#ifdef FG_DEBUG
+      printf("fg: tcsetpgrp(%d)\n",(int)pg);
+#endif
       tcsetpgrp(0,pg);
-      if(kill(-pg,SIGCONT)==-1)
-	 kill(pg,SIGCONT);
    }
+   cont();
 }
 
 void FgData::Bg()
 {
    if(old_pgrp)
    {
+#ifdef FG_DEBUG
+      printf("bg: tcsetpgrp(%d)\n",(int)old_pgrp);
+#endif
       tcsetpgrp(0,old_pgrp);
       old_pgrp=0;
    }
