@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.h,v 1.56 2000/03/21 14:27:02 lav Exp $ */
+/* $Id: ftpclass.h,v 1.67 2000/08/29 21:52:16 lav Exp $ */
 
 #ifndef FTPCLASS_H
 #define FTPCLASS_H
@@ -28,6 +28,10 @@
 
 #include "NetAccess.h"
 
+#ifdef USE_SSL
+# include <openssl/ssl.h>
+#endif
+
 class Ftp : public NetAccess
 {
    static Ftp *ftp_chain;
@@ -38,21 +42,13 @@ class Ftp : public NetAccess
       EOF_STATE,	   // control connection is open, idle state
       INITIAL_STATE,	   // all connections are closed
       CONNECTING_STATE,	   // we are connecting the control socket
+      CONNECTED_STATE,	   // just after connect
       WAITING_STATE,	   // we're waiting for a response with data
       ACCEPTING_STATE,	   // we're waiting for an incoming data connection
       DATA_OPEN_STATE,	   // data connection is open, for read or write
-      FATAL_STATE,	   // fatal error occured (e.g. command not implemented)
-      NO_FILE_STATE,	   // file access is impossible - no such file
-      NO_HOST_STATE,	   // host not found or not specified
-      STORE_FAILED_STATE,  // STOR failed - you have to reput
-      LOGIN_FAILED_STATE,  // login failed due to invalid password
-      SYSTEM_ERROR_STATE,  // system error occured, errno saved to saved_errno
-      LOOKUP_ERROR_STATE,  // unsuccessful host name lookup
-/*      ERROR_STATE,	   // see error_code.*/
       CWD_CWD_WAITING_STATE,  // waiting until 'CWD $cwd' finishes
       USER_RESP_WAITING_STATE,// waiting until 'USER $user' finishes
-      DATASOCKET_CONNECTING_STATE,   // waiting for data_sock to connect
-      COPY_FAILED
+      DATASOCKET_CONNECTING_STATE  // waiting for data_sock to connect
    };
 
    enum response
@@ -101,6 +97,10 @@ class Ftp : public NetAccess
       CHECK_PASS,	// check response for PASS
       CHECK_PASS_PROXY,	// check response for PASS sent to proxy
       CHECK_TRANSFER	// generic check for transfer
+#ifdef USE_SSL
+      ,CHECK_AUTH_TLS,
+      CHECK_PROT_P
+#endif
    };
 
    struct expected_response;
@@ -108,7 +108,6 @@ class Ftp : public NetAccess
    struct expected_response
    {
       int   expect;
-      int   fail_state;
       check_case_t check_case;
       bool  log_resp;
       char  *path;
@@ -124,9 +123,9 @@ class Ftp : public NetAccess
 
    void	 LogResp(const char *line);
 
-   void  AddResp(int exp,int fail,check_case_t ck=CHECK_NONE,bool log=false);
+   void  AddResp(int exp,check_case_t ck=CHECK_NONE,bool log=false);
    void  SetRespPath(const char *p);
-   int   CheckResp(int resp);
+   void  CheckResp(int resp);
    int	 ReplyLogPriority(int code);
    void  PopResp();
    void	 EmptyRespQueue();
@@ -135,18 +134,18 @@ class Ftp : public NetAccess
    int	 RespQueueSize() { return RQ_tail-RQ_head; }
    expected_response *FindLastCWD();
 
-   int	 RestCheck(int,int);
-   int   NoFileCheck(int,int);
-   int	 TransferCheck(int,int);
-   int	 LoginCheck(int,int);
-   int	 NoPassReqCheck(int,int);
-   int	 proxy_LoginCheck(int,int);
-   int	 proxy_NoPassReqCheck(int,int);
+   void	 RestCheck(int);
+   void  NoFileCheck(int);
+   void	 TransferCheck(int);
+   void	 LoginCheck(int);
+   void	 NoPassReqCheck(int);
+   void	 proxy_LoginCheck(int);
+   void	 proxy_NoPassReqCheck(int);
    char *ExtractPWD();
-   int	 CatchDATE(int,int);
-   int	 CatchDATE_opt(int,int);
-   int	 CatchSIZE(int,int);
-   int	 CatchSIZE_opt(int,int);
+   void	 CatchDATE(int);
+   void	 CatchDATE_opt(int);
+   void	 CatchSIZE(int);
+   void	 CatchSIZE_opt(int);
    int	 Handle_PASV();
    int	 Handle_EPSV();
 
@@ -155,6 +154,23 @@ class Ftp : public NetAccess
    int   control_sock;
    int   data_sock;
    int	 aborted_data_sock;
+   bool	 quit_sent;
+   bool	 fixed_pasv; // had to fix PASV address.
+
+#ifdef USE_SSL
+   SSL	 *control_ssl;
+   bool	 control_ssl_connected;
+   SSL	 *data_ssl;
+   bool	 data_ssl_connected;
+   char	 prot;	  // current data protection scheme 'C'lear or 'P'rivate
+   void	 BlockOnSSL(SSL*);
+protected:
+   bool	 ftps;	  // ssl and prot='P' by default (port 990)
+private:
+   bool	 auth_tls_sent;
+#else
+   static const bool ftps=false; // for convenience
+#endif
 
    /* type of transfer: TYPE_A or TYPE_I */
    int   type;
@@ -166,9 +182,12 @@ class Ftp : public NetAccess
    int   resp_size;
    int	 resp_alloc;
    char  *line;
+   char  *all_lines;
+
+   bool	 eof;
 
    time_t   nop_time;
-   long	    nop_offset;
+   off_t    nop_offset;
    int	    nop_count;
 
    time_t   stat_time;
@@ -187,8 +206,11 @@ class Ftp : public NetAccess
 
    void  SendCmd(const char *cmd,int len=-1);
    void  SendCmd2(const char *cmd,const char *f);
+   void  SendCmd2(const char *cmd,int v);
    void  SendUrgentCmd(const char *cmd);
    int	 FlushSendQueue(bool all=false);
+   void	 SendArrayInfoRequests();
+   void	 SendSiteIdle();
 
    int	 ReceiveResp();
 	 // If a response is received, it checks it for accordance with
@@ -219,9 +241,9 @@ class Ftp : public NetAccess
    bool  mdtm_supported;
    bool  size_supported;
    bool  site_chmod_supported;
-   long	 last_rest;
+   off_t last_rest;
 
-   int	 StateToError();
+   void	 SetError(int code,const char *mess=0);
 
    int	 addr_received;	// state of PASV
 
@@ -261,6 +283,7 @@ private:
    bool	copy_done;
    bool	copy_connection_open;
    bool copy_allow_store;
+   bool copy_failed;
 
    bool use_stat;
    int  stat_interval;
@@ -288,7 +311,7 @@ public:
    {
       if(control_sock==-1)
 	 return 0;
-      if(state==CONNECTING_STATE)
+      if(state==CONNECTING_STATE || quit_sent)
 	 return 1;
       return 2;
    }
@@ -352,15 +375,14 @@ public:
 	 copy_addr_valid=true;
 	 return true;
       }
-   bool CopyFailed() { return state==COPY_FAILED; }
+   bool CopyFailed() { return copy_failed; }
    bool RestartFailed() { return flags&NOREST_MODE; }
    bool IsPassive() { return flags&PASSIVE_MODE; }
-   void SetPos(long p) { pos=real_pos=p; }
    bool IsCopyPassive() { return copy_passive; }
    void CopyAllowStore()
       {
 	 SendCmd2("STOR",file);
-	 AddResp(RESP_TRANSFER_OK,STORE_FAILED_STATE,CHECK_TRANSFER);
+	 AddResp(RESP_TRANSFER_OK,CHECK_TRANSFER);
 	 copy_allow_store=true;
       }
    bool CopyStoreAllowed() { return copy_allow_store; }
@@ -372,4 +394,17 @@ public:
       }
 };
 
+class FtpS : public Ftp
+{
+public:
+   FtpS();
+   FtpS(const FtpS *);
+   ~FtpS();
+
+   const char *GetProto() { return "ftps"; }
+
+   FileAccess *Clone() { return new FtpS(this); }
+   static FileAccess *New();
+};
+
 #endif /* FTPCLASS_H */
