@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp.cc,v 1.35 2000/04/12 18:51:40 lav Exp $ */
+/* $Id: lftp.cc,v 1.38 2000/07/08 15:03:42 lav Exp $ */
 
 #include <config.h>
 
@@ -37,13 +37,6 @@
 #endif
 
 #include "xalloca.h"
-#include "ftpclass.h"
-#ifndef MODULE_PROTO_FILE
-# include "LocalAccess.h"
-#endif
-#ifndef MODULE_PROTO_HTTP
-# include "Http.h"
-#endif
 #include "xmalloc.h"
 #include "alias.h"
 #include "CmdExec.h"
@@ -88,17 +81,23 @@ class ReadlineFeeder : public CmdFeeder
    bool ctty:1;
    bool add_newline:1;
    char *to_free;
+   int eof_count;
 public:
    ReadlineFeeder()
    {
       tty=isatty(0);
       ctty=(tcgetpgrp(0)!=(pid_t)-1);
       to_free=0;
+      eof_count=0;
    }
    virtual ~ReadlineFeeder()
    {
       xfree(to_free);
    }
+   bool RealEOF()
+   {
+      return !tty || eof_count>3;
+   }
 
    const char *NextCmd(class CmdExec *exec,const char *prompt)
    {
@@ -159,6 +158,11 @@ public:
 	 else if(cmd_buf==0 && exec->interactive)
 	    puts("exit");
 	 xmalloc_register_block(cmd_buf);
+
+	 if(cmd_buf==0)
+	    eof_count++;
+	 else
+	    eof_count=0;
       }
       else
       {
@@ -291,19 +295,11 @@ int   main(int argc,char **argv)
    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);
 
-   ResMgr::ClassInit(); // resources must be inited before other classes
+   // order is significant.
    SignalHook::ClassInit();
+   ResMgr::ClassInit();
    Resolver::ClassInit();
-
-#ifndef MODULE_PROTO_FILE
-   LocalAccess::ClassInit();
-#endif
-#ifndef MODULE_PROTO_HTTP
-   Http::ClassInit();
-#endif
-#ifndef MODULE_PROTO_FTP
-   Ftp::ClassInit();
-#endif
+   FileAccess::ClassInit();
 
    lftp_readline_init();
 
