@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp_rl.h,v 1.2 1999/07/10 13:36:49 lav Exp $ */
+/* $Id: lftp_rl.h,v 1.3 2000/02/12 19:24:39 lav Exp $ */
 
 #ifndef LFTP_RL_H
 #define LFTP_RL_H
@@ -27,7 +27,7 @@ CDECL_BEGIN
 
 void lftp_readline_init (void);
 int  lftp_history_expand (const char *, char **);
-const char *lftp_readline (const char *prompt);
+char *lftp_readline (const char *prompt);
 void lftp_add_history_nodups(const char *cmd);
 void lftp_rl_clear(void);
 void lftp_rl_redisplay_maybe(void);
