@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: mgetJob.cc,v 1.14 2000/03/24 18:53:24 lav Exp $ */
+/* $Id: mgetJob.cc,v 1.16 2000/07/18 22:49:10 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -147,9 +147,9 @@ int mgetJob::Do()
    {
       if(mkdir_job->Done())
       {
+	 RemoveWaiting(mkdir_job);
 	 Delete(mkdir_job);
 	 mkdir_job=0;
-	 waiting=0;
       }
       else
 	 return STALL;
@@ -171,9 +171,9 @@ int mgetJob::Do()
 	 {
 	    mkdir_job=new mkdirJob(Clone(),mkdir_args);
 	    mkdir_job->BeQuiet();
-	    waiting=mkdir_job;
-	    waiting->SetParentFg(this);
-	    waiting->cmdline=mkdir_args->Combine();
+	    AddWaiting(mkdir_job);
+	    mkdir_job->SetParentFg(this);
+	    mkdir_job->cmdline=mkdir_args->Combine();
 	    // don't delete mkdir_args; -- mkdirJob does it
 	    mkdir_args=0;
 	 }
@@ -203,20 +203,18 @@ int mgetJob::Do()
 
    m=MOVED;
 
-   char **files=rg->GetResult();
-   if(!files)
+   FileSet *files=rg->GetResult();
+   if(files->get_fnum()==0)
    {
       fprintf(stderr,_("%s: no files found\n"),rg->glob->GetPattern());
       goto next;
    }
-   while(*files)
-   {
-      char *src=*files;
+   do {
+      char *src=files->curr()->name;
       args->Append(src);
       make_directory(src);
       args->Append(output_file_name(src,0,!reverse,output_dir,make_dirs));
-      files++;
-   }
+   } while(files->next());
    goto next;
 }
 
