@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.19 2000/03/18 23:46:15 lav Exp $ */
+/* $Id: misc.cc,v 1.22 2000/06/09 00:17:58 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -365,14 +365,15 @@ const char *format_perms(int p)
    return s;
 }
 
+const char month_names[][4]={
+   "Jan","Feb","Mar","Apr","May","Jun",
+   "Jul","Aug","Sep","Oct","Nov","Dec",
+   ""
+};
 int parse_month(const char *m)
 {
-   static const char *months[]={
-      "Jan","Feb","Mar","Apr","May","Jun",
-      "Jul","Aug","Sep","Oct","Nov","Dec",0
-   };
-   for(int i=0; months[i]; i++)
-      if(!strcasecmp(months[i],m))
+   for(int i=0; month_names[i][0]; i++)
+      if(!strcasecmp(month_names[i],m))
 	 return(i%12);
    return -1;
 }
@@ -403,7 +404,7 @@ int guess_year(int month,int day,int hour,int minute)
       year--;
    return year;
 }
-int percent(long offset,long size)
+int percent(off_t offset,off_t size)
 {
    if(offset>=size)
       return 100;
