@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.h,v 1.12 2000/03/18 23:46:15 lav Exp $ */
+/* $Id: misc.h,v 1.15 2000/06/09 00:17:58 lav Exp $ */
 
 #ifndef MISC_H
 #define MISC_H
@@ -55,7 +55,7 @@ void  truncate_file_tree(const char *dir);
 // returns 0 on error.
 char *xgetcwd();
 
-int percent(long offset,long size);
+int percent(off_t offset,off_t size);
 
 #define find_char(buf,len,ch) ((const char *)memchr(buf,ch,len))
 
@@ -63,6 +63,8 @@ int percent(long offset,long size);
 #define HOUR   (60*MINUTE)
 #define DAY    (24*HOUR)
 
+extern const char month_names[][4];
+
 int parse_month(const char *);
 int parse_perms(const char *);
 const char *format_perms(int p);
