@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: pgetJob.cc,v 1.22.2.1 2000/07/12 12:00:07 lav Exp $ */
+/* $Id: pgetJob.cc,v 1.27 2000/09/15 18:08:01 lav Exp $ */
 
 #include <config.h>
 #include "pgetJob.h"
@@ -49,19 +49,27 @@ int pgetJob::Do()
    if(cp->Done())
       RemoveBackupFile();
 
+   if(chunks_done && chunks && cp->GetPos()>=chunks[0]->start)
+   {
+      cp->SetRange(0,cp->GetPos());
+      cp->Resume();
+      free_chunks();
+      m=MOVED;
+   }
+
    if(chunks==0 || cp->GetPos()<chunks[0]->start)
    {
       cp->Resume();
       m=super::Do(); // it can call NextFile.
    }
-   else
+   else if(chunks)
       cp->Suspend();
 
    if(Done() || chunks_done)
       return m;
 
-   long offset=cp->GetPos();
-   long size=cp->GetSize();
+   off_t offset=cp->GetPos();
+   off_t size=cp->GetSize();
 
    if(chunks==0)
    {
@@ -78,7 +86,7 @@ int pgetJob::Do()
       cp->GetPut()->NeedSeek(); // seek before writing
 
       /* initialize chunks */
-      long chunk_size=(size-offset)/max_chunks;
+      off_t chunk_size=(size-offset)/max_chunks;
       if(chunk_size<0x10000)
 	 chunk_size=0x10000;
       num_of_chunks=(size-offset)/chunk_size-1;
@@ -88,7 +96,7 @@ int pgetJob::Do()
 	 return MOVED;
       }
       chunks=(ChunkXfer**)xmalloc(sizeof(*chunks)*num_of_chunks);
-      long curr_offs=size;
+      off_t curr_offs=size;
       for(int i=num_of_chunks; i-->0; )
       {
 	 const char *name=cp->GetName();
@@ -105,13 +113,13 @@ int pgetJob::Do()
 	 chunks[i]=NewChunk(s,name,local,curr_offs-chunk_size,curr_offs);
 	 chunks[i]->SetParentFg(this,false);
 	 chunks[i]->cmdline=(char*)xmalloc(7+2*20+1);
-	 sprintf(chunks[i]->cmdline,"\\chunk %ld-%ld",
-				    curr_offs-chunk_size,curr_offs-1);
+	 sprintf(chunks[i]->cmdline,"\\chunk %lld-%lld",
+	    (long long)(curr_offs-chunk_size),(long long)(curr_offs-1));
 	 curr_offs-=chunk_size;
       }
       xfree(cp->cmdline);
       cp->cmdline=(char*)xmalloc(7+2*20+1);
-      sprintf(cp->cmdline,"\\chunk %ld-%ld",0L,chunks[0]->start-1);
+      sprintf(cp->cmdline,"\\chunk 0-%lld",(long long)(chunks[0]->start-1));
       m=MOVED;
    }
 
@@ -119,17 +127,13 @@ int pgetJob::Do()
    chunks_done=true;
    total_xferred=MIN(offset,chunks[0]->start);
    total_xfer_rate=cp->GetRate();
-   total_eta=-1;
-   if(total_xfer_rate<1)
-      total_eta=-2;
+
+   off_t rem=chunks[0]->start-cp->GetPos();
+   if(rem<=0)
+      total_eta=0;
    else
-   {
-      long rem=chunks[0]->start-cp->GetPos();
-      if(rem<=0)
-	 total_eta=0;
-      else
-	 total_eta=cp->GetETA(rem);
-   }
+      total_eta=cp->GetETA(rem);
+
    for(int i=0; i<num_of_chunks; i++)
    {
       if(chunks[i]->Error())
@@ -142,11 +146,11 @@ int pgetJob::Do()
 	 if(chunks[i]->GetPos()>=chunks[i]->start)
 	    total_xferred+=MIN(chunks[i]->GetPos(),chunks[i]->limit)
 			   -chunks[i]->start;
-	 if(total_eta!=-2)
+	 if(total_eta>=0)
 	 {
 	    long eta=chunks[i]->GetETA();
 	    if(eta<0)
-	       total_eta=-2;
+	       total_eta=-1;
 	    else if(eta>total_eta)
 	       total_eta=eta;	// total eta is the maximum.
 	 }
@@ -165,20 +169,13 @@ int pgetJob::Do()
       return MOVED;
    }
 
-   if(chunks_done && cp->GetPos()>=chunks[0]->start)
-   {
-      cp->SetRange(0,cp->GetPos());
-      cp->Resume();
-      free_chunks();
-      return MOVED;
-   }
-
    return m;
 }
 
 // xgettext:c-format
-#define PGET_STATUS \
-   _("`%s', got %lu of %lu (%d%%) %s%s"),name,total_xferred,size, \
+static const char pget_status_format[]=N_("`%s', got %lld of %lld (%d%%) %s%s");
+#define PGET_STATUS _(pget_status_format),name, \
+   (long long)total_xferred,(long long)size, \
    percent(total_xferred,size),Speedometer::GetStrS(total_xfer_rate), \
    cp->GetETAStrSFromTime(total_eta)
 
@@ -194,7 +191,7 @@ void pgetJob::ShowRunStatus(StatusLine *s)
       return;
 
    const char *name=cp->SqueezeName(s->GetWidthDelayed()-40);
-   long size=cp->GetSize();
+   off_t size=cp->GetSize();
    s->Show(PGET_STATUS);
 }
 
@@ -226,7 +223,7 @@ void  pgetJob::PrintStatus(int verbose)
 
    printf("\t");
    const char *name=cp->GetName();
-   long size=cp->GetSize();
+   off_t size=cp->GetSize();
    printf(PGET_STATUS);
    printf("\n");
 }
@@ -267,7 +264,7 @@ void pgetJob::NextFile()
 }
 
 pgetJob::ChunkXfer *pgetJob::NewChunk(FileAccess *session,const char *remote,
-				       FDStream *local,long start,long limit)
+				       FDStream *local,off_t start,off_t limit)
 {
    FileCopyPeerFDStream
 	       	*dst_peer=new FileCopyPeerFDStream(local,FileCopyPeer::PUT);
@@ -287,7 +284,7 @@ pgetJob::ChunkXfer *pgetJob::NewChunk(FileAccess *session,const char *remote,
 }
 
 pgetJob::ChunkXfer::ChunkXfer(FileCopy *c1,const char *name,
-			      long s,long lim)
+			      off_t s,off_t lim)
    : CopyJob(c1,name,"pget")
 {
    start=s;
